// Trilobytes - Star Knightess Aura Event Shift/
// TLB_SKAEventShift.js
//=============================================================================

window.Imported = window.Imported || {};
window.Imported.TLB_SKAEventShift = true;

window.TLB = window.TLB || {};
TLB.SKAEventShift = TLB.SKAEventShift || {};

/*:
 * @target MZ
 * @plugindesc This plugin adds a notetag for events which shifts their
 * position on map load by a defined number of tiles. Its primary intent is for
 * bridge lower railings.
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help
 * ============================================================================
 * Introduction
 * ============================================================================
 *
 * This is an ad hoc plugin which adds a new event notetag.
 *
 * ============================================================================
 * Plugin Parameters
 * ============================================================================
 *
 * All parameters are explained in their respective description field.
 *
 * ============================================================================
 * Plugin Commands
 * ============================================================================
 *
 * None
 *
 * ============================================================================
 * How to Use
 * ============================================================================
 *
 * There are three notetags you can add to an event:
 *
 * <shift:x,y>
 * <shift x:number>
 * <shift y:number>
 *
 * The tag with both coordinates will take precedence over the other two in the
 * event that both are present on the same event for some reason.
 *
 * Use a negative shift to move left or up, and a positive to move right or
 * down.
 *
 * If a shift value has a decimal value, this will offset the event's screen
 * position on its new tile by that value. For example <shift x:0.25> will
 * simply move the event half a tile to the right.
 *
 * ============================================================================
 * Compatibility
 * ============================================================================
 *
 * There shouldn't be any compatibility issues with non-menu plugins.
 *
 * ============================================================================
 * Terms of Use
 * ============================================================================
 *
 * Copyright 2022 Auradev
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * ============================================================================
 *
 */

//----------------------------------------------------------------------------
//
// Parameter conversion
//
//----------------------------------------------------------------------------

const _Game_Event_initialize = Game_Event.prototype.initialize;
Game_Event.prototype.initialize = function(mapId, eventId) {
	const meta = $dataMap.events[eventId].meta;
	const shiftAll = meta?.["shift"];
	const shift = shiftAll
		? shiftAll.split(",")
		: [meta?.["shift x"], meta?.["shift y"]];

	const xShift = shift[0] ? Number(shift[0]) : 0;
	const yShift = shift[1] ? Number(shift[1]) : 0;

	// For every integer shift, we shift the actual location
	this._xLocationShift = parseInt(xShift);
	this._yLocationShift = parseInt(yShift);

	// For fractional shifts, we only shift the rendering position
	this._xScreenShift = xShift - this._xLocationShift;
	this._yScreenShift = yShift - this._yLocationShift;

	_Game_Event_initialize.call(this, mapId, eventId);
}

TLB.SKAEventShift.Game_Event_locate = Game_Event.prototype.locate;
Game_Event.prototype.locate = function(x, y) {
	// Check for existence of shift properties for backward compatibility
	TLB.SKAEventShift.Game_Event_locate.call(this, x + (this._xLocationShift || 0), y + (this._yLocationShift || 0));
};

TLB.SKAEventShift.Game_Event_screenX = Game_Event.prototype.screenX;
Game_Event.prototype.screenX = function() {
	const screenX = TLB.SKAEventShift.Game_Event_screenX.call(this);
	if (this._xScreenShift) {
		const xOffset = $gameMap.tileWidth() * this._xScreenShift;
		return screenX + xOffset;
	} else {
		return screenX;
	}
};

TLB.SKAEventShift.Game_Event_screenY = Game_Event.prototype.screenY;
Game_Event.prototype.screenY = function() {
	const screenY = TLB.SKAEventShift.Game_Event_screenY.call(this);
	if (this._yScreenShift) {
		const yOffset = $gameMap.tileWidth() * this._yScreenShift;
		return screenY + yOffset;
	} else {
		return screenY;
	}
};
