//=============================================================================
// RPG Maker MZ - Compendium
// ----------------------------------------------------------------------------
// (C)2021 aura-dev
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// 1.0.0 2021/11/27
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Compendium
 * @author aura-dev
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help compendium.js
 *
 * This plugin provides a UI for tracking various information about the game state.
 *
 * Added tags:
 * <compendium_show_skills_conditions: return [CONDITION1, ..., CONDITION_N]
 * - Displays a skill only of CONDITION is mapped. The ID of the CONDITION must match with the
 * action pattern ID of the skill. E.g. the skill on the first position in an enemy's action pattern
 * list is checked for CONDITION1.
 *
 * @command addBeastiaryEntry
 * @text Add Beastiary Entry
 * @desc Adds a new entry to the beastiary
 *
 * @arg enemy
 * @type enemy
 * @text enemy
 * @desc The enemy
 *
 * @command revealStat
 * @text Reveal Stat
 * @desc Reveals the stat of an enemy
 *
 * @arg enemy
 * @type enemy
 * @text enemy
 * @desc The enemy
 *
 * @arg stat
 * @type number
 * @text stat
 * @desc The ID of the stat
 *
 * @command revealAffinity
 * @text Reveal Affinity
 * @desc Reveals the affinity of an enemy
 *
 * @arg enemy
 * @type enemy
 * @text enemy
 * @desc The enemy
 *
 * @arg elementID
 * @type number
 * @text elementID
 * @desc The ID of the element of the affinity
 *
 * @command revealSkill
 * @text Reveal Skill
 * @desc Reveals the skill of an enemy
 *
 * @arg enemy
 * @type enemy
 * @text enemy
 * @desc The enemy
 *
 * @arg skillID
 * @type skill
 * @desc The ID of the element of the affinity
 *
 * @command revealTrait
 * @text Reveal Trait
 * @desc Reveals the trait of an enemy
 *
 * @arg enemy
 * @type enemy
 * @text enemy
 * @desc The enemy
 *
 * @arg trait
 * @type string
 * @test trait
 * @desc The trait
 *
 * @command revealAll
 * @text Reveal All
 * @desc Reveals all data except for secret traits
 *
 * @arg enemy
 * @type enemy
 * @text enemy
 * @desc The enemy
 *
 * @command addBossKill
 * @text Add Defeated Boss
 * @desc Add 1 to the Tracked Number of Defeated Bosses
 *
 * @arg enemy
 * @type enemy
 * @text enemy
 * @desc The enemy just defeated
 * 
 * @param compendiumCommand_cte
 * @text Compendium Command
 * @desc The text to use for the Compendium command.
 * @default Compendium
 *
 * @param bestiaryCommand_cte
 * @text Bestiary Command
 * @desc The text to use for the Bestiary command.
 * @default Bestiary
 * 
 * @param socialCommand_cte
 * @text Social Command
 * @desc The text to use for the Social command.
 * @default Social
 * 
 * @param teleportCommand_cte
 * @text Teleport Command
 * @desc The text to use for the Teleport command.
 * @default Teleport
 * 
 * @param locationsCommand_cte
 * @text Locations Command
 * @desc The text to use for the Locations command.
 * @default Locations
 * 
 * @param drainText_cte
 * @text Drain Text
 * @desc The text to use for element drain.
 * @default DR
 * 
 * @param resText_cte
 * @text Resistance Text
 * @desc The text to use for element resistance.
 * @default RS
 * 
 * @param weakText_cte
 * @text Weakness Text
 * @desc The text to use for element weakness.
 * @default WK
 * 
 * @param skillsText_cte
 * @text Skills Text
 * @desc The text to use for the Skills list.
 * @default Skills
 * 
 * @param traitsText_cte
 * @text Traits Text
 * @desc The text to use for the Traits list.
 * @default Traits
 * 
 * @param defeatedText_cte
 * @text Defeated Text
 * @desc The text to use for the number of an enemy defeated.
 * @default Defeated
 * 
 */

// ---------------------------------------------------------------------------------
// OBJECTS
// ---------------------------------------------------------------------------------

window.AuraMZ = window.AuraMZ || {};
AuraMZ.Compendium = AuraMZ.Compendium || {};

AuraMZ.Compendium.Params = PluginManager.parameters("compendium");

Object.defineProperties(AuraMZ.Compendium, {
	"compendiumCommand": { get: () => TextManager.message(`auramz/compendium-compendiumCommand_cte`) || AuraMZ.Compendium.Params.compendiumCommand_cte },
	"bestiaryCommand": { get: () => TextManager.message(`auramz/compendium-bestiaryCommand_cte`) || AuraMZ.Compendium.Params.bestiaryCommand_cte },
	"socialCommand": { get: () => TextManager.message(`auramz/compendium-socialCommand_cte`) || AuraMZ.Compendium.Params.socialCommand_cte },
	"teleportCommand": { get: () => TextManager.message(`auramz/compendium-teleportCommand_cte`) || AuraMZ.Compendium.Params.teleportCommand_cte },
	"locationsCommand": { get: () => TextManager.message(`auramz/compendium-locationsCommand_cte`) || AuraMZ.Compendium.Params.locationsCommand_cte },
	"drainText": { get: () => TextManager.message(`auramz/compendium-drainText_cte`) || AuraMZ.Compendium.Params.drainText_cte },
	"resText": { get: () => TextManager.message(`auramz/compendium-resText_cte`) || AuraMZ.Compendium.Params.resText_cte },
	"weakText": { get: () => TextManager.message(`auramz/compendium-weakText_cte`) || AuraMZ.Compendium.Params.weakText_cte },
	"skillsText": { get: () => TextManager.message(`auramz/compendium-skillsText_cte`) || AuraMZ.Compendium.Params.skillsText_cte },
	"traitsText": { get: () => TextManager.message(`auramz/compendium-traitsText_cte`) || AuraMZ.Compendium.Params.traitsText_cte },
	"defeatedText": { get: () => TextManager.message(`auramz/compendium-defeatedText_cte`) || AuraMZ.Compendium.Params.defeatedText_cte }
});

DataManager._databaseFiles.push({ name: "$dataCompendium", src: "Compendium.json" });

// Global variable for accessing bestiary data
window.$gameBeastiary = null;

// Plain data container for containing the registered quests
class Game_Beastiary {
	constructor() {
		this._data = [];
	}
}

class BeastiaryEntry {
	constructor() {
		this._enemyID = 0;
		this._knownStats = [];
		this._knownAffinities = [];
		this._knownSkills = [];
		this._knownTraits = [];
		this._numKilled = 0;
	}
}

class SocialContact {
	constructor(name, tags, characterName, characterIndex, direction, pattern) {
		this.name = name;
		this.tags = tags;
		this.characterName = characterName;
		this.characterIndex = characterIndex;
		this.direction = (direction === "") ? 0 : direction;
		this.pattern = (pattern === "") ? 1 : pattern;
	}
}


// Service class for managing the compendium
class CompendiumManager {
	static isCompendiumEnabled() {
		return eval($dataCompendium.enabledCondition);
	}
}

// Service class for manipulating beastiary data
class BeastiaryManager {
	static addBeastiaryEntry(enemyID) {
		if (!$gameBeastiary._data[enemyID]) {
			const beastiaryEntry = new BeastiaryEntry();
			beastiaryEntry._enemyID = enemyID;
			$gameBeastiary._data[enemyID] = beastiaryEntry;
		}
	}

	static getOrCreateBeastiaryEntry(enemyID) {
		if (!$gameBeastiary._data[enemyID]) {
			BeastiaryManager.addBeastiaryEntry(enemyID);
		}

		return $gameBeastiary._data[enemyID];
	}

	static stats() {
		return [
			{ name: TextManager.hp, param: 0 },
			{ name: TextManager.mp, param: 1 },
			{ name: TextManager.param(2), param: 2 },
			{ name: TextManager.param(4), param: 4 },
			{ name: TextManager.param(3), param: 3 },
			{ name: TextManager.param(5), param: 5 },
			{ name: TextManager.param(6), param: 6 },
			{ name: TextManager.param(7), param: 7 },
		];
	}

	static isKnown(enemyID) {
		const entry = $gameBeastiary._data[enemyID];
		return entry;
	}

	static isKnownStat(enemyID, stat) {
		for (const enemy of $dataEnemies) {
			if (enemy && enemy.name == $dataEnemies[enemyID].name) {
				const entry = $gameBeastiary._data[enemy.id];
				if (entry?._knownStats[stat]) {
					return true;
				}
			}
		}

		return false;
	}

	static isKnownAffinity(enemyID, element) {
		for (const enemy of $dataEnemies) {
			if (enemy && enemy.name == $dataEnemies[enemyID].name) {
				const entry = $gameBeastiary._data[enemy.id];
				if (entry?._knownAffinities[element]) {
					return true;
				}
			}
		}

		return false;
	}

	static isKnownSkill(enemyID, skillID) {
		for (const enemy of $dataEnemies) {
			if (enemy && enemy.name == $dataEnemies[enemyID].name) {
				const entry = $gameBeastiary._data[enemy.id];
				if (entry?._knownSkills.includes(parseInt(skillID))) {
					return true;
				}
			}
		}

		return false;
	}

	static isKnownTrait(enemyID, trait) {
		for (const enemy of $dataEnemies) {
			if (enemy && enemy.name == $dataEnemies[enemyID].name) {
				const entry = $gameBeastiary._data[enemy.id];
				if (entry?._knownTraits.includes(trait)) {
					return true;
				}
			}
		}

		return false;
	}

	static getNumKilled(enemyID) {
		const entry = $gameBeastiary._data[enemyID];
		if (entry) {
			return entry._numKilled;
		}
	}

	static revealStat(enemyID, stat) {
		const entry = BeastiaryManager.getOrCreateBeastiaryEntry(enemyID);
		entry._knownStats[stat] = true;
	}

	static revealAllStats(enemyID) {
		for (const stat of BeastiaryManager.stats()) {
			BeastiaryManager.revealStat(enemyID, stat.param);
		}
	}

	static revealAffinity(enemyID, element) {
		const entry = BeastiaryManager.getOrCreateBeastiaryEntry(enemyID);
		entry._knownAffinities[element] = true;
	}

	static revealAllAffinities(enemyID) {
		for (const element in $dataSystem.elements) {
			BeastiaryManager.revealAffinity(enemyID, element);
		}
	}

	static revealSkill(enemyID, skillID) {
		const entry = BeastiaryManager.getOrCreateBeastiaryEntry(enemyID);
		if (!entry._knownSkills.includes(skillID)) {
			entry._knownSkills.push(skillID);
		}
	}

	static revealAllSkills(enemyID) {
		const skills = $dataEnemies[enemyID].actions
			.filter(action => action != null)
			.map(action => action.skillId);
		for (const skillID of skills) {
			BeastiaryManager.revealSkill(enemyID, skillID);
		}
	}

	static revealTrait(enemyID, trait) {
		const entry = BeastiaryManager.getOrCreateBeastiaryEntry(enemyID);
		if (!entry._knownTraits.includes(trait)) {
			entry._knownTraits.push(trait);
		}
	}

	static revealAll(enemyID) {
		BeastiaryManager.revealAllStats(enemyID);
		BeastiaryManager.revealAllAffinities(enemyID);
		BeastiaryManager.revealAllSkills(enemyID);
	}

	static addKill(enemyID) {
		const entry = BeastiaryManager.getOrCreateBeastiaryEntry(enemyID);
		entry._numKilled++;
	}



	static getEntries(includeTraits, excludeTraits) {
		return $gameBeastiary._data.filter(enemy => enemy && includeTraits.every(trait => $dataEnemies[enemy._enemyID].meta[trait]) && excludeTraits.every(trait => !$dataEnemies[enemy._enemyID].meta[trait]));
	}
}

// Scene providing the Compendium
class Scene_Compendium extends Scene_MenuBase {
	create() {
		super.create();
		this.createCategoryWindow();
		this.createPages();
	}
	
	createPages() {
		this._bestiaryWindow = this.createPage(Window_EnemyList, this.onBestiaryPageOk);
		this._enemyDetailWindow = this.createDetailPage(Window_EnemyDetail, this._bestiaryWindow);
		this._socialWindow = this.createPage(Window_SocialList, this.onSocialPageOk);
		this._socialDetailWindow = this.createDetailPage(Window_SocialDetail, this._socialWindow);
		if ($dataCompendium.addLocations) {
			this._locationWindow = this.createPage(Window_LocationList, this.onLocationPageOk);
			this._locationDetailWindow = this.createDetailPage(Window_LocationDetail, this._locationWindow);
			this._locationDetailWindow.setHandler("teleport", this.onTeleportOk.bind(this));
			this._locationDetailWindow.setHandler("social", this.onSocialOk.bind(this));
		}
	}

	createCategoryWindow() {
		const rect = this.categoryWindowRect();
		this._categoryWindow = new Window_CompendiumCategory(rect);
		this._categoryWindow.setHandler("ok", this.onCategoryOk.bind(this));
		this._categoryWindow.setHandler("cancel", this.popScene.bind(this));
		this.addWindow(this._categoryWindow);
	}

	categoryWindowRect() {
		const wx = 0;
		const wy = this.mainAreaTop();
		const ww = Graphics.boxWidth;
		const wh = this.calcWindowHeight(1, true);
		return new Rectangle(wx, wy, ww, wh);
	}

	createPage(windowClass, okHandler) {
		const rect = this.pageWindowRect();
		const window = new windowClass(rect);
		window.setHandler("ok", okHandler.bind(this));
		window.setHandler("cancel", this.onPageCancel.bind(this));
		window.refresh();
		this.addWindow(window);
		this._categoryWindow.addPageWindow(window);
		window.hide();
		return window;
	}

	pageWindowRect() {
		const wx = 0;
		const wy = this._categoryWindow.y + this._categoryWindow.height;
		const ww = Graphics.boxWidth / 3;
		const wh = this.mainAreaHeight();
		return new Rectangle(wx, wy, ww, wh);
	}

	createDetailPage(windowClass, parent) {
		const rect = this.detailWindowRect(parent);
		const detailPage = new windowClass(rect);
		detailPage.setHandler("cancel", this.onDetailCancel.bind(this, detailPage, parent));
		detailPage.refresh();
		this.addWindow(detailPage);
		parent.addPageWindow(detailPage);
		detailPage.hide();
		return detailPage;
	}

	detailWindowRect(parent) {
		const wx = parent.width;
		const wy = parent.y;
		const ww = 2 * Graphics.boxWidth / 3;
		const wh = parent.height;
		return new Rectangle(wx, wy, ww, wh);
	}

	onCategoryOk() {
		const window = this._categoryWindow._mapCategoryToPage[this._categoryWindow.index()];
		window.activate();
		window.selectFirst();
	};

	onPageCancel() {
		this._socialWindow.deselect();
		this._locationWindow.deselect();
		this._locationDetailWindow.contents.clear();
		this._bestiaryWindow.deselect();
		this._categoryWindow.activate();
	}

	onDetailCancel(window, parent) {
		window.deselect();
		parent.activate();
	}

	onBestiaryPageOk() {
		const item = this._bestiaryWindow.item();
		if (!item.name) {
			this._bestiaryWindow.flipExpansionState(item);
			this._bestiaryWindow.refresh();
		}

		this._bestiaryWindow.activate();
	}

	onSocialPageOk() {
		this._socialDetailWindow.activate();
		this._socialDetailWindow.select(0);
	}

	onLocationPageOk() {
		this._locationDetailWindow.activate();
		this._locationDetailWindow.select(0);
	}

	onTeleportOk() {
		this.activateTeleport(this._locationDetailWindow);
	}

	activateTeleport(sourceWindow) {
		const location = sourceWindow._location;
		if (location && location.teleportCE) {
			$gameTemp.reserveCommonEvent(parseInt(location.teleportCE));
			SceneManager.goto(Scene_Map);
		} else {
			sourceWindow.activate();
		}
	}

	onSocialOk() {
		const location = this._locationDetailWindow._location;
		if (location) {
			this._locationDetailWindow.deselect();
			this._locationWindow.deselect();
			this._categoryWindow.select(this._categoryWindow.findSymbol("social"));
			this._socialWindow.activate();
			this._socialWindow.select(this._socialWindow._data.findIndex(data => data.name_cte == location.name_cte));
		} else {
			this._locationDetailWindow.activate();
		}
	}
}

// ---------------------------------------------------------------------------------
// WINDOWS
// ---------------------------------------------------------------------------------

// Window that manages the tabs for the different quest states
class Window_CompendiumCategory extends Window_HorzCommand {
	constructor(rect) {
		super(rect);
		this._mapCategoryToPage = [];
	}

	makeCommandList() {
		this.addCommand(AuraMZ.Compendium.bestiaryCommand, "bestiary", $gameBeastiary._data.length > 0);
		this.addCommand(AuraMZ.Compendium.socialCommand, "social");
		if ($dataCompendium.addLocations) {
			this.addCommand(AuraMZ.Compendium.locationsCommand, "locations");
		}
	}

	update() {
		super.update();

		for (const window of this._mapCategoryToPage) {
			window.hide();
		}

		const index = this.index() == -1 ? 0 : this.index();
		this._mapCategoryToPage[index].show();
	}


	addPageWindow(window) {
		this._mapCategoryToPage.push(window);
	}
}

class Window_Compendium extends Window_Selectable {
	constructor(rect) {
		super(rect);
		this._data = [];
		this._mapPageToWindow = [];
	}
	
	itemAt(index) {
		return index >= 0 ? this._data[index] : null;
	}
	
	item() {
		return this._data[this.index()];
	}
	
	selectFirst() {
		this.forceSelect(0);
	}
	
	maxCols() {
		return 1;
	}

	colSpacing() {
		return 16;
	}

	maxItems() {
		return this._data.length;
	}
	
	update() {
		super.update();
		
		for (const window of this._mapPageToWindow) {
			window.hide();
		}		
	}
}

class Window_CompendiumDetail extends Window_Selectable {
	maxCols() {
		return 1;
	}

	colSpacing() {
		return 16;
	}

	maxItems() {
		return 1;
	}	
}

// Manages a list of enemies
class Window_EnemyList extends Window_Compendium {
	constructor(rect) {
		super(rect);
		this._enemyCategories = $dataCompendium.races_cte;
		this._expandedCategories = [];
	}
	
	update() {
		super.update();

		if (this.visible && this._mapPageToWindow[0]) {
			this._mapPageToWindow[0].show();
			const item = this.item();
			if (item?.name) {
				this._mapPageToWindow[0].setEnemy(this.item());
			}
		}
	}

	hasCategory(enemy, category) {
		return enemy.meta[category.toLowerCase()] == "true";
	}

	isExpanded(category) {
		return this._expandedCategories[category];
	}

	flipExpansionState(category) {
		this._expandedCategories[category] = !this._expandedCategories[category];
	}

	makeEnemies() {
		this._data = this.createEnemyList();
	}

	createEnemyList() {
		const array = [];
		const enemyList = $dataEnemies.filter((enemy, index) => enemy && $gameBeastiary._data[enemy.id] && $dataEnemies.findIndex(e => e && $gameBeastiary._data[e.id] && e.name === enemy.name) === index);
		for (const category of this._enemyCategories) {
			const knownEnemies = enemyList.filter(enemy => this.hasCategory(enemy, category));

			if (knownEnemies.length > 0) {
				this.addEnemyCategory(array, category);
				if (this.isExpanded(category)) {
					const sortedKnownEnemies = knownEnemies.sort((enemy1, enemy2) => $dataEnemies[enemy1.id].name.localeCompare($dataEnemies[enemy2.id].name));
					array.push(...sortedKnownEnemies);
				}
			}
		}
		return array;
	}

	addEnemyCategory(array, category) {
		array.push(category);
	}

	getLocalisedRace(race) {
		const raceIndex = $dataCompendium.races_cte.findIndex(dbRace => dbRace === race);
		if (raceIndex > -1 && window.CTE) {
			return TextManager.localize(race, "compendium", "races_cte", raceIndex);
		}
		return race;
	}

	getNonLocalisedRace(localisedRace) {
		const cteRaces = window.CTE?.getData("compendium", "races_cte");
		if (cteRaces) {
			const raceIndex = Object.values(cteRaces).findIndex(locRace => locRace === localisedRace);
			if (raceIndex > -1) {
				return $dataCompendium.races_cte[raceIndex];
			}
		}
		return localisedRace;
	}

	refresh() {
		this.makeEnemies();
		super.refresh();
	}

	getLabel(item) {
		if (item.name) {
			return "  " + item.name;
		} else {
			const sign = this.isExpanded(item) ? "-" : "+";
			return sign + " " + this.getLocalisedRace(item);
		}
	}

	drawItem(index) {
		const item = this.itemAt(index);
		if (item) {
			const rect = this.itemLineRect(index);
			this.drawText(this.getLabel(item), rect.x, rect.y, rect.width);
		}
	}

	addPageWindow(window) {
		this._mapPageToWindow.push(window);
	}
}

// This window manages the detail screen of an enemy page
class Window_EnemyDetail extends Window_CompendiumDetail {
	constructor(rect) {
		super(rect);

		this._enemySprite = new Sprite();
		this._enemySprite.anchor.set(0.5);
		this._enemySprite.move(this.width * 0.2, this.height * 0.2);
		this.addInnerChild(this._enemySprite);
	}

	setEnemy(enemy) {
		this._enemy = enemy;
		if (enemy) {
			this._gameEnemy = new Game_Enemy(enemy.id, -1, -1);

			const bitmap = ImageManager.loadSvEnemy(enemy.battlerName);
			this._enemySprite.bitmap = bitmap;
			this._enemySprite.show();
			if (bitmap && !bitmap.isReady()) {
				bitmap.addLoadListener(this.initSprite.bind(this));
			} else {
				this.initSprite();
			}
		} else {
			this._enemySprite.hide();
			this.refresh();
		}
	}

	initSprite() {
		const scaleX = this.width * 0.4 / this._enemySprite._bitmap.width;
		const scaleY = this.height * 0.4 / this._enemySprite._bitmap.height;
		const scale = Math.min(scaleX, scaleY, 1);
		this._enemySprite.scale.x = scale;
		this._enemySprite.scale.y = scale;
		this._enemySprite.setHue(this._enemy.battlerHue);

		this.refresh();
	}

	makeStat(name, stat) {
		const value = BeastiaryManager.isKnownStat(this._enemy.id, stat) ? this._gameEnemy.param(stat) : "?";
		return { name: name, value: value };
	}

	makeStats() {
		return BeastiaryManager.stats().map(stat => this.makeStat(stat.name, stat.param));
	}

	drawStats() {
		const stats = this.makeStats();
		const padding = this.width * 0.04;
		const offsetStats = this.width * 0.45;
		const statsWidth = this.width * 0.225;
		let offset = padding;
		for (let i = 0; i < stats.length; i += 2) {
			const stat1 = stats[i];
			this.drawTextEx("\\C[16]" + stat1.name + "\\C[0]", offsetStats, offset, statsWidth);
			this.drawText(stat1.value, offsetStats, offset, statsWidth, "right");

			const stat2 = stats[i + 1];
			this.drawTextEx("\\C[16]" + stat2.name + "\\C[0]", offsetStats + statsWidth + padding, offset, statsWidth);
			this.drawText(stat2.value, offsetStats + statsWidth + padding, offset, statsWidth, "right");

			offset += this.lineHeight();
		}
	}

	getAffinity(element) {
		if (!BeastiaryManager.isKnownAffinity(this._enemy.id, element)) {
			return "?";
		}

		if (Imported.TLB_SKAElementDrain) {
			const elementId = TLB.SKAElementDrain.getDrainElement(this._enemy);
			if (elementId === element) {
				return AuraMZ.Compendium.drainText;
			}
		}
		const elementRate = this._gameEnemy.elementRate(element);
		return AuraMZ.Compendium.getElementRateLabel(elementRate, "-");
	}

	drawAffinities() {
		const mapElementIDToIconID = [null, 77, 64, null, 66, 67, 68, 69, 70, 71];
		const offsetY = this.height * 0.4;
		let offsetX = 0;
		for (let i = 0; i < $dataSystem.elements.length; ++i) {
			const iconID = mapElementIDToIconID[i];
			if (iconID != null) {
				this.drawIcon(iconID, offsetX, offsetY);
				const affinity = this.getAffinity(i);
				this.drawText(affinity, offsetX, offsetY + ImageManager.iconHeight, ImageManager.iconWidth, "center");
				offsetX += this.width / 8;
			}
		}
	}

	makeEnemyActions(enemy) {
		const compendium_show_skills_conditions = enemy.meta.compendium_show_skills_conditions;
		if (compendium_show_skills_conditions) {
			if (!enemy.compendium_show_skills_conditions) {
				enemy.compendium_show_skills_conditions = new Function(compendium_show_skills_conditions);
			}

			const conditions = enemy.compendium_show_skills_conditions.call(this);
			return enemy.actions.flatMap((action, index) => conditions[index] ? [action] : []);
		} else {
			return enemy.actions;
		}
	}

	makeSkills() {
		const skills = $dataEnemies
			.filter(enemy => enemy && enemy.name == this._enemy.name)
			.flatMap(enemy => this.makeEnemyActions(enemy))
			.filter(action => action != null)
			.map(action => $dataSkills[action.skillId]);
		return [...new Set(skills)];
	}

	drawSkills() {
		const text = `\\C[16][${AuraMZ.Compendium.skillsText}]\\C[0] ` + this.makeSkills()
			.map(skill => BeastiaryManager.isKnownSkill(this._enemy.id, skill.id)
				? "\\I[" + skill.iconIndex + "]" + skill.name
				: "?")
			.join(", ");
		const offsetY = this.height * 0.4 + ImageManager.iconHeight + this.lineHeight();
		this.drawTextEx(this.formatDetail(text), 0, offsetY, this.width);
	}

	makeTraits() {
		const traits = [];
		for (const [key, value] of Object.entries(this._enemy.meta)) {
			const normalizedTrait = key.charAt(0).toUpperCase() + key.slice(1);
			if ($dataCompendium.traits_cte.includes(normalizedTrait) || $dataCompendium.races_cte.includes(normalizedTrait)) {
				if (value == "true" || BeastiaryManager.isKnownTrait(this._enemy.id, key)) {
					traits.push(this.getLocalisedTrait(normalizedTrait));
				} else if (value == "secret") {
					traits.push("?");
				} else {
					traits.push(normalizedTrait + " " + eval(value));
				}
			}

		}
		return traits;
	}

	getLocalisedTrait(trait) {
		if (window.CTE) {
			for (const db of ["traits_cte", "races_cte"]) {
				const index = $dataCompendium[db].indexOf(trait);
				if (index >= 0) {
					const data = CTE.getData("compendium", db);
					if (data?.[index]) return data[index];
				}
			}
		}

		return trait;
	}

	drawTraits() {
		const text = `\\C[16][${AuraMZ.Compendium.traitsText}]\\C[0] ` + this.makeTraits().join(", ");
		const offsetY = this.height * 0.4 + ImageManager.iconHeight + this.lineHeight() * 4;
		this.drawTextEx(this.formatDetail(text), 0, offsetY, this.width);
	}

	makeKilledBosses() {
		let total = parseInt(this._enemy.meta.num);
		if (total > 0) {
			const numKilled = BeastiaryManager.getNumKilled(this._enemy.id) || 0;

			// if user has killed all, show total, otherwise a "?"
			if (!this.showTotalBosses(numKilled, total)) {
				total = "?";
			}

			return [numKilled, total];
		}
		return null;
	}

	showTotalBosses(killed, total) {
		return killed === total;
	}

	drawKilledBosses() {
		const text = this.makeKilledBosses();
		const offsetY = this.height * 0.4 + ImageManager.iconHeight + this.lineHeight() * 6;
		if (text) {
			this.drawTextEx(`\\C[16]${AuraMZ.Compendium.defeatedText}\\C[0] ${text[0]}/${text[1]}`, 0, offsetY, this.width);
		}
	}

	drawAllItems() {
		if (this._enemy) {
			this.drawStats();
			this.drawAffinities();
			this.drawSkills();
			this.drawTraits();
			this.drawKilledBosses();
		}
	}

	formatDetail(data) {
		const DETAIL_PADDING = 10;
		let detail = "";
		const lines = data.split("\n");
		lines.forEach((line) => {
			const words = line.split(' ');
			let x = 0;
			words.forEach((word) => {
				const wordWithWhitespace = word + ' ';
				let wordForWidth = wordWithWhitespace;
				let iconWidth = 0;

				if (wordWithWhitespace.startsWith("\\")) {
					const indexMetaDataEnd = wordWithWhitespace.indexOf("]");
					wordForWidth = wordWithWhitespace.substring(indexMetaDataEnd + 1);
					if (wordWithWhitespace[1] == "I") {
						iconWidth = ImageManager.iconWidth;
					}
				}

				const wordWidth = this.textWidth(wordForWidth) + iconWidth;
				if (x + wordWidth + DETAIL_PADDING > this.width) {
					x = 0;
					detail += "\n";
				}

				detail += wordWithWhitespace;
				x += wordWidth;
			});

			detail += "\n";
		});

		return detail;
	}
}

// Manages a list of social contacts
class Window_SocialList extends Window_Compendium {
	update() {
		super.update();

		if (this.visible && this._mapPageToWindow[0]) {
			this._mapPageToWindow[0].show();
			const item = this.item();
			if (item) {
				this._mapPageToWindow[0].setLocation(item);
			}
		}
	}

	makeLocations() {
		return $dataCompendium.locations?.filter(location => AuraMZ.Compendium.doesLocationHaveUnlockedCharacters(location.name_cte)) || [];
	}

	refresh() {
		this._data = this.makeLocations();
		super.refresh();
	}

	drawItem(index) {
		const item = this.itemAt(index);
		if (item) {
			const rect = this.itemLineRect(index);
			const locIndex = $dataCompendium.locations.findIndex(loc => loc.name_cte === item.name_cte);
			this.drawText(window.CTE ? TextManager.localize(item.name_cte, "compendium", "locations", locIndex, "name_cte") : item.name_cte, rect.x, rect.y, rect.width);
		}
	}

	addPageWindow(window) {
		this._mapPageToWindow.push(window);
	}
}

// This window manages the detail screen of a social page
class Window_SocialDetail extends Window_CompendiumDetail {
	constructor(rect) {
		super(rect);
		this._location = "";
		this._data = [];
	}

	maxItems = Window_Compendium.prototype.maxItems;

	itemAt = Window_Compendium.prototype.itemAt;

	item = Window_Compendium.prototype.item;

	itemHeight = function() {
		return 2 * Window_Scrollable.prototype.itemHeight.call(this) + 8;
	};

	selectFirst = Window_Compendium.prototype.selectFirst;

	setLocation(location) {
		if (location !== this._location) {
			this._location = location;
			this.makeSocialList();
			this.refresh();
		}
	}

	makeTags(locIndex, charIndex, character) {
		const $ = {};
		const tags = [];
		eval(character.tagSetup);
		for (let i = 0; i < (character.tagList || []).length; i++) {
			const tagProgression = character.tagList[i];
			if (!tagProgression.groupCondition || eval(tagProgression.groupCondition) == true) {
				for (let j = 0; j < tagProgression.tags.length; j++) {
					const tag = tagProgression.tags[j];
					if (!tag.condition || eval(tag.condition)) {
						tag.text_cte = window.CTE ? TextManager.localize(tag.text_cte, "compendium", "locations", locIndex, "characters", charIndex, "tagList", i, "tags", j, "text_cte") : tag.text_cte;
						tags.push(tag);
						const currentIndex = tagProgression.tags.indexOf(tag);
						const nextIndex = currentIndex + 1;
						if (!tagProgression.tags[nextIndex] || tagProgression.tags[nextIndex].identifier !== tag.identifier) break;
					}
				}
			}
		}
		return tags;
	}

	drawCharacterWithPattern(
		characterName, characterIndex, x, y, direction = 0, pattern = 1
	) {
		const bitmap = ImageManager.loadCharacter(characterName);
		const big = ImageManager.isBigCharacter(characterName);
		const pw = bitmap.width / (big ? 3 : 12);
		const ph = bitmap.height / (big ? 4 : 8);
		const n = big ? 0 : characterIndex;
		const sx = ((n % 4) * 3 + pattern) * pw;
		const sy = Math.floor(n / 4) * 4 * ph + (direction * ph);
		this.contents.blt(bitmap, sx, sy, pw, ph, x - pw / 2, y - ph);
	};

	makeSocialContacts() {
		const contacts = [];
		const makeSocialContactCharacter = function(character, $) {
			for (const image of character.charImages) {
				if (!image.condition || eval(image.condition)) {
					return image;
				}
			}
		};
		for (let i = 0; i < $dataCompendium.locations.length; i++) {
			const location = $dataCompendium.locations[i];
			for (let j = 0; j < location.characters?.length; j++) {
				const character = location.characters[j];
				const $ = {};
				eval(character.tagSetup);
				const appearsElsewhere = (character.alternateLocations || []).some(altLoc => altLoc.name != this._location && eval(altLoc.condition));
				const appearsHere = location.name_cte == this._location.name_cte || (character.alternateLocations || []).some(altLoc => altLoc.name == this._location.name_cte && eval(altLoc.condition));
				const includeChar = !appearsElsewhere && appearsHere;
				if (includeChar) {
					const tags = this.makeTags(i, j, character);
					const { characterName, characterIndex, direction, pattern } = makeSocialContactCharacter(character, $);
					contacts.push(new SocialContact(eval(window.CTE ? TextManager.localize(character.name_cte, "compendium", "locations", i, "characters", j, "name_cte") : character.name_cte), tags, characterName, characterIndex, direction, pattern));
				}
			}
		}
		return contacts;
	}

	makeSocialList() {
		this._data = this.makeSocialContacts();
	}

	drawItem(index) {
		const item = this.itemAt(index);
		if (item) {
			const rect = this.itemLineRect(index);

			const bitmap = ImageManager.loadCharacter(item.characterName);
			if (bitmap && !bitmap.isReady()) {
				bitmap.addLoadListener(this.drawCharacterWithPattern.bind(this, item.characterName, item.characterIndex, rect.x + 24, rect.y + 48, item.direction, item.pattern));
			} else {
				this.drawCharacterWithPattern(item.characterName, item.characterIndex, rect.x + 24, rect.y + 48, item.direction, item.pattern);
			}

			const name = "\\c[16][" + item.name + "]\\c[0]\\FS[18]\n";
			const tags = item.tags.map(tag => eval(tag.text_cte)).join(", ");
			this.drawTextEx(name + tags, rect.x + 48, rect.y - 16, rect.width);
		}
	}
}

// Manages a list of locations
class Window_LocationList extends Window_Compendium {	
	update() {
		super.update();

		if (this.visible && this._mapPageToWindow[0]) {
			this._mapPageToWindow[0].show();
			const item = this.item();
			if (item) {
				this._mapPageToWindow[0].setLocation(item);
			}
		}
	}

	makeLocations() {
		return $dataCompendium.locations?.filter(location => location.showInCompendium && (!location.showCondition || eval(location.showCondition))) || [];
	}

	refresh() {
		this._data = this.makeLocations();
		super.refresh();
	}

	drawItem(index) {
		const item = this.itemAt(index);
		if (item) {
			const rect = this.itemLineRect(index);
			this.drawText(window.CTE?.getData("compendium", "locations", index, "name_cte") || item.name_cte, rect.x + 30, rect.y, rect.width);
			this.drawShrine(item, 10, rect.y + 6);
		}
	}

	drawShrine(item, x, y) {
		if (item.shrineCondition) {
			const activated = eval(item.shrineCondition);
			const image = "!$BigUtil1";
			const bmp = ImageManager.loadCharacter(image);
			bmp.addLoadListener(() => {
				const pw = bmp.width / 3;
				const ph = bmp.height / 4;
				const sx = 0;
				const sy = (activated ? 3 : 0) * ph;
				this.contents.blt(bmp, sx, sy, pw, ph, x, y, pw / 4, ph / 4)
			}, this)
		}
	}

	addPageWindow(window) {
		this._mapPageToWindow.push(window);
	}
}

// This window manages the detail screen of a location page
class Window_LocationDetail extends Window_HorzCommand {

	constructor(rect) {
		super(rect);
		this._location = null;
		this.select(-1);
		this.deactivate();
	}

	makeCommandList() {
		this.addCommand(AuraMZ.Compendium.teleportCommand, "teleport", !!(this._location && this._location.shrineCondition && eval(this._location.shrineCondition)));
		this.addCommand(AuraMZ.Compendium.socialCommand, "social", !!(this._location && AuraMZ.Compendium.doesLocationHaveUnlockedCharacters(this._location.name_cte)));
	};

	itemHeight = function() {
		return 2 * Window_Scrollable.prototype.itemHeight.call(this) + 8;
	};

	selectFirst = Window_Compendium.prototype.selectFirst;

	setLocation(location) {
		this._location = location;
		this.refresh();
	}

	itemRect(index) {
		const rect = super.itemRect(index);
		rect.y += 450;
		return rect;
	}

	drawAllItems() {
		const location = this._location;
		if (location) {
			this.drawThumbnail();
			this.drawDescription();
			const topIndex = this.topIndex();
			for (let i = 0; i < this.maxVisibleItems(); i++) {
				const index = topIndex + i;
				if (index < this.maxItems()) {
					this.drawItemBackground(index);
					this.drawItem(index);
				}
			}
		}
	}

	drawThumbnail() {
		if (!this._worldmapThumbnail && $worldMap) {
			const worldmapId = 4;
			this._worldmapThumbnail = new Spriteset_Worldmap_Thumbnail($worldMap, worldmapId);
			this.addChildAt(this._worldmapThumbnail, 1);
			this._worldmapThumbnail.filterArea = new Rectangle(598, 148, 332, 188);
		} else if (!this._worldmapThumbnail) {
			setTimeout(() => this.drawThumbnail(), 100);
		}

		if (this._worldmapThumbnail) {
			setTimeout(() => { if (this._worldmapThumbnail) this._worldmapThumbnail.visible = true }, 100);
			const gameMap = this._worldmapThumbnail.gameMap();
			gameMap.setDisplayPos(parseInt(this._location.thumbnailX) || 0, parseInt(this._location.thumbnailY) || 0);
			gameMap._displayX = parseInt(this._location.thumbnailX) || 0;
			gameMap._displayY = parseInt(this._location.thumbnailY) || 0;
		}
	}
	
	drawDescription() {
		const location = this._location;
		const locIndex = $dataCompendium.locations.findIndex(loc => loc == location);
		let paraY = 200;
		for (let i = 0; i < (location.descriptions || []).length; i++) {
			const paragraph = location.descriptions[i];
			if (!paragraph.condition || eval(paragraph.condition)) {
				this.changePaintOpacity(true);
				for (let j = 0; j < paragraph.description_cte.length; j++) {
					const line = paragraph.description_cte[j];
					this.drawTextEx(window.CTE ? TextManager.localize(line, "compendium", "locations", locIndex, "descriptions", i, "description_cte", j) : line, 0, paraY);
					paraY += this.textSizeEx(line).height;
				}
			}
		}	
	}
}

class Spriteset_Worldmap_Thumbnail extends Spriteset_Map {
	constructor(worldMap, worldMapId) {
		// Can't use callWithWorldMap since it uses this
		// but usage of this is forbidden before calling super
		const dataMap = $dataMap;
		const gameMap = $gameMap;
		$dataMap = worldMap;
		const gameWorldMap = new Game_Map();
		gameWorldMap.setup(worldMapId);
		$gameMap = gameWorldMap;
		super();
		$dataMap = dataMap;
		$gameMap = gameMap;

		this._worldMap = worldMap;
		this._gameWorldMap = gameWorldMap;
		this.visible = false;
	}

	callWithWorldMap(func) {
		const dataMap = $dataMap;
		const gameMap = $gameMap;
		$dataMap = this._worldMap;
		$gameMap = this._gameWorldMap;

		func.call(this);

		$dataMap = dataMap;
		$gameMap = gameMap;
	}

	gameMap() {
		return this._gameWorldMap;
	}

	update() {
		this.callWithWorldMap(super.update);
	}

	updateBaseFilters() {
		// We don't want the thumbnail to reflect current screen tones.
	}

	createCharacters() {
		this._characterSprites = [];
		for (const event of $gameMap.events()) {
			this._characterSprites.push(new Sprite_Character(event));
		}
		for (const vehicle of $gameMap.vehicles()) {
			this._characterSprites.push(new Sprite_Character(vehicle));
		}
		for (const sprite of this._characterSprites) {
			this._tilemap.addChild(sprite);
		}
	};
}

(($) => {
	const PLUGIN_NAME = "compendium";

	$.hasTag = function(tags, tagId) {
		return tags.some(tag => tag.identifier == tagId);
	};

	$.doesLocationHaveUnlockedCharacters = function(locationName) {
		for (const location of $dataCompendium.locations) {
			for (const character of location.characters || []) {
				const $ = {};
				eval(character.tagSetup);
				const unlockedAlternate = (character.alternateLocations || []).some(altLoc => altLoc.name == locationName && eval(altLoc.condition));
				const unlockedHere = location.name_cte == locationName;
				if (unlockedAlternate || unlockedHere) return true;
			}
		}
		return false;
	};

	const _DataManager_loadDatabase = DataManager.loadDatabase;
	DataManager.loadDatabase = function() {
		_DataManager_loadDatabase.call(this);
		const worldmapId = 4;
		const filename = "Map%1.json".format(worldmapId.padZero(3));
		DataManager.loadDataFile("$worldMap", filename);
	};

	// Initialize the data container for bestiary
	const _DataManager_createGameObjects = DataManager.createGameObjects;
	DataManager.createGameObjects = () => {
		_DataManager_createGameObjects();
		window.$gameBeastiary = new Game_Beastiary();
	};

	// Ensure that the current quest data is saved
	const _DataManager_makeSaveContents = DataManager.makeSaveContents;
	DataManager.makeSaveContents = () => {
		const contents = _DataManager_makeSaveContents();
		contents.beastiary = $gameBeastiary;
		return contents;
	};

	// Ensure that the current quest data is loaded
	const _DataManager_extractSaveContents = DataManager.extractSaveContents;
	DataManager.extractSaveContents = (contents) => {
		_DataManager_extractSaveContents(contents);
		if (contents.beastiary) {
			window.$gameBeastiary = contents.beastiary;
		}
	};

	// Command that adds a new beastiary entry
	PluginManager.registerCommand(PLUGIN_NAME, "addBeastiaryEntry", args => {
		BeastiaryManager.addBeastiaryEntry(args.enemyID);
	});

	// Command that reveals a stat
	PluginManager.registerCommand(PLUGIN_NAME, "revealStat", args => {
		BeastiaryManager.revealStat(args.enemy, args.stat);
	});

	// Command that reveals an affinity
	PluginManager.registerCommand(PLUGIN_NAME, "revealAffinity", args => {
		BeastiaryManager.revealAffinity(args.enemy, args.elementID);
	});

	// Command that reveals a skill
	PluginManager.registerCommand(PLUGIN_NAME, "revealSkill", args => {
		BeastiaryManager.revealSkill(args.enemy, parseInt(args.skillID));
	});

	// Command that reveals everything except secrets
	PluginManager.registerCommand(PLUGIN_NAME, "revealAll", args => {
		BeastiaryManager.addBeastiaryEntry(args.enemy);
		BeastiaryManager.revealAll(args.enemy);
	});

	// Command that reveals a trait
	PluginManager.registerCommand(PLUGIN_NAME, "revealTrait", args => {
		BeastiaryManager.revealTrait(args.enemy, args.trait);
	});

	// Command that adds a kill
	PluginManager.registerCommand(PLUGIN_NAME, "addBossKill", args => {
		BeastiaryManager.addKill(args.enemy);
	});

	// Tracked Forms of an enemy
	let trackedForms = [];

	function trackForm(enemyId) {
		trackedForms.push(enemyId);
	}

	function revealTrackedForms() {
		for (const enemyId of trackedForms) {
			BeastiaryManager.revealAll(enemyId);
		}
		clearTrackedForms();
	}

	function clearTrackedForms() {
		trackedForms = [];
	}

	// Reveal all data of an enemy upon defeating it
	const _BattleManager_processVictory = BattleManager.processVictory;
	BattleManager.processVictory = function() {
		_BattleManager_processVictory.call(this);
		if (CompendiumManager.isCompendiumEnabled()) {
			const appearedMembers = $gameTroop.troop().members.filter(member => !member.hidden);
			for (const member of appearedMembers) {
				BeastiaryManager.addBeastiaryEntry(member.enemyId);
				BeastiaryManager.revealAll(member.enemyId);
			}
			revealTrackedForms();
		}
	};

	// Reveal the beastiary entry of an enemy when escaping
	const _BattleManager_processEscape = BattleManager.processEscape;
	BattleManager.processEscape = function() {
		_BattleManager_processEscape.call(this);
		if (CompendiumManager.isCompendiumEnabled()) {
			for (const member of $gameTroop.troop().members.filter(member => !member.hidden)) {
				BeastiaryManager.addBeastiaryEntry(member.enemyId);
				BeastiaryManager.revealAll(member.enemyId);
			}
			clearTrackedForms();
		}
	};

	// Reveal the beastiary entry of an enemy when losing
	const _BattleManager_processDefeat = BattleManager.processDefeat;
	BattleManager.processDefeat = function() {
		_BattleManager_processDefeat.call(this);
		if (CompendiumManager.isCompendiumEnabled()) {
			for (const member of $gameTroop.troop().members.filter(member => !member.hidden)) {
				BeastiaryManager.addBeastiaryEntry(member.enemyId);
				BeastiaryManager.revealAll(member.enemyId);
			}
			clearTrackedForms();
		}
	};

	// Reveal the elemental affinity when an enemy takes damage
	const _Game_Action_executeDamage = Game_Action.prototype.executeDamage;
	Game_Action.prototype.executeDamage = function(target, value) {
		_Game_Action_executeDamage.call(this, target, value);

		if (CompendiumManager.isCompendiumEnabled() && target._enemyId) {
			BeastiaryManager.revealAffinity(target._enemyId, this.item().damage.elementId);
		}
	}

	// Reveal skill when enemy uses it
	const _Game_Enemy_performAction = Game_Enemy.prototype.performAction;
	Game_Enemy.prototype.performAction = function(action) {
		_Game_Enemy_performAction.call(this, action);
		if (CompendiumManager.isCompendiumEnabled()) {
			BeastiaryManager.revealSkill(this._enemyId, action._item._itemId);
		}
	};

	// Track forms when an enemy transforms into a different enemy
	const _Game_Enemy_transform = Game_Enemy.prototype.transform;
	Game_Enemy.prototype.transform = function(enemyId) {
		const originalEnemyId = this._enemyId;
		_Game_Enemy_transform.call(this, enemyId);
		if (CompendiumManager.isCompendiumEnabled() && originalEnemyId !== this._enemyId) {
			BeastiaryManager.addBeastiaryEntry(this._enemyId);
			trackForm(this._enemyId);
		}
	};

	const _Game_Enemy_appear = Game_Enemy.prototype.appear;
	Game_Enemy.prototype.appear = function() {
		_Game_Enemy_appear.call(this);
		if (CompendiumManager.isCompendiumEnabled()) {
			BeastiaryManager.addBeastiaryEntry(this._enemyId);
			trackForm(this._enemyId);
		}
	};

	const _AuraMZ_startNewGamePlus = AuraMZ.startNewGamePlus;
	AuraMZ.startNewGamePlus = function() {
		const oldGameBeastiary = $gameBeastiary;
		_AuraMZ_startNewGamePlus.call();
		window.$gameBeastiary = oldGameBeastiary;

		// reset number of killed enemies to 0
		for (const entry of $gameBeastiary._data) {
			if (entry) {
				entry._numKilled = 0;
			}
		}
	}

	// Registers the compendium entry in the main menu
	const _Window_MenuCommand_addMainCommands = Window_MenuCommand.prototype.addMainCommands;
	Window_MenuCommand.prototype.addMainCommands = function() {
		_Window_MenuCommand_addMainCommands.call(this);
		const enabled = CompendiumManager.isCompendiumEnabled();
		this.addCommand($.compendiumCommand, "compendium", enabled);
	};

	// Registers the handler to open the quest log in the main menu
	const _Scene_Menu_createCommandWindow = Scene_Menu.prototype.createCommandWindow;
	Scene_Menu.prototype.createCommandWindow = function() {
		_Scene_Menu_createCommandWindow.call(this);
		this._commandWindow.setHandler("compendium", this.commandCompendium.bind(this));
	};

	Scene_Menu.prototype.commandCompendium = function() {
		SceneManager.push(Scene_Compendium);
	};

	$.getElementRateLabel = function(elementRate, normalText) {
		if (elementRate < 1) {
			return AuraMZ.Compendium.resText;
		} else if (elementRate > 1) {
			return AuraMZ.Compendium.weakText;
		} else {
			return normalText;
		}
	}
})(AuraMZ.Compendium);
