// Trilobytes - Controller Functions/
// controller_functions.js
//=============================================================================

window.Imported = window.Imported || {};
window.Imported.ControllerFunctions = true;

/*:
 * @target MZ
 * @plugindesc This plugin adds functions that deal with controllers.
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help
 * ============================================================================
 * Introduction
 * ============================================================================
 *
 * This is an ad hoc plugin which adds controller-related functions.
 *
 * ============================================================================
 * Plugin Parameters
 * ============================================================================
 *
 * None
 *
 * ============================================================================
 * How to use
 * ============================================================================
 *
 * Plug and play.
 *
 * ============================================================================
 * Compatibility
 * ============================================================================
 *
 * There shouldn't be any compatibility issues.
 *
 * ============================================================================
 * Terms of Use
 * ============================================================================
 *
 * Copyright 2025 Trihan
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * @param disconnectMessage_cte
 * @text Disconnect Message
 * @desc The text to use when a controller is disconnected.
 * @default Controller has been disconnected.\nPlease reconnect or press any key\nor mouse button to continue.
 * 
 */

window.AuraMZ = window.AuraMZ || {};
AuraMZ.ControllerFunctions = {};

Object.defineProperties(AuraMZ.ControllerFunctions, {
  "disconnectMessage": { get: () => TextManager.message(`auramz/controller-functions-disconnectMessage_cte`) || PluginManager.parameters("controller_functions").disconnectMessage_cte }
});

class Window_ControllerDisconnected extends Window_Base {
	constructor(rect) {
		super(rect);
		this.refresh();
		this.hide();
		this._controllerDisconnected = false;
	}

	onControllerConnect() {
		this._controllerDisconnected = false;
		this.hide();
	}

	onControllerDisconnect() {
		this._controllerDisconnected = true;
		this.show();
	}

	updateOpen() {
		super.updateOpen()
		if (this.isOpen() && this._controllerDisconnected) {
			Graphics.stopGameLoop();
		}
	};

	refresh() {
		this.contents.clear();
		this.drawIcon(333, 0, 0);
		this.drawTextExCenter(AuraMZ.ControllerFunctions.disconnectMessage, 0, 0, this.contentsWidth());
	}
}

(() => {  
  window.addEventListener("gamepadconnected", (e) => {
    if (!Graphics._app?._ticker.started) {
      Graphics.startGameLoop();
    }
    if (SceneManager._scene?._cdWindow) SceneManager._scene._cdWindow.onControllerConnect();
    SceneManager._scene?.onControllerConnect?.();
    document.body.style.cursor = "none"
  });
  
  window.addEventListener("keydown", (e) => {
    if (!Graphics._app?._ticker.started) {
      Graphics.startGameLoop();
      if (SceneManager._scene?._cdWindow) SceneManager._scene._cdWindow.onControllerConnect();
      SceneManager._scene?.onControllerDisconnect?.();
    }
  });
  
  window.addEventListener("mousedown", (e) => {
    if (!Graphics._app?._ticker.started) {
      Graphics.startGameLoop();
      if (SceneManager._scene?._cdWindow) SceneManager._scene._cdWindow.onControllerConnect();
      SceneManager._scene?.onControllerDisconnect?.();
      document.body.style.cursor = ""
    }
  });
  
  window.addEventListener("gamepaddisconnected", (e) => {
    const scene = SceneManager._scene;
    if (!scene._cdWindow && scene._windowLayer) {
      const rect = scene.controllerDisconnectedRect();
      scene.createControllerDisconnectedWindow(rect);
    }
    if (scene._cdWindow) scene._cdWindow.onControllerDisconnect();
    SceneManager._scene?.onControllerDisconnect?.();
    document.body.style.cursor = ""
  });
  
  
  
  Scene_Base.prototype.createControllerDisconnectedWindow = function() {
    const rect = this.controllerDisconnectedRect();
    this._cdWindow = new Window_ControllerDisconnected(rect);
    this.addWindow(this._cdWindow);
    if (SceneManager._scene instanceof Scene_Skill || SceneManager._scene instanceof Scene_Status) {
      this.sortableChildren = true;
      this.children.filter(child => child === this._windowLayer)[0]._zIndex = 100;
    } else {
      this._windowLayer.sortableChildren = true;
      this._cdWindow._zIndex = 100;
    }
  };
  
  Scene_Base.prototype.controllerDisconnectedRect = function() {
    return new Rectangle(Graphics.width / 2 - 406, Graphics.height / 2 - 170, 430, 140);
  }
})();