//=============================================================================
// RPG Maker MZ - Counterskill
// ----------------------------------------------------------------------------
// (C)2021 aura-dev
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// 1.0.0 2025/08/02
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Counterskill Tag
 * @author aura-dev
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help advanced_states.js
 *
 * Adds tags for giving actors and enemies a "counterskill".
 * A counterskill is executed if the counterskill condition is true.
 *
 * Added tags:
 * <counter_skill: NUMBER>
 * - ID of the conunter skill.
 * <counter_skill_condition: CONDITION>
 * - A counterskill is activated if the target is attacked while CONDITION is true.
 */

window.AuraMZ = window.AuraMZ || {};

(() => {
	// Checks for counterattack skills on damage and memorizes the skill and item
	// id of the last skill that hit the target
	const _Game_Action_executeDamage = Game_Action.prototype.executeDamage;
	Game_Action.prototype.executeDamage = function(target, value) {
		_Game_Action_executeDamage.call(this, target, value);

		for (const traitObject of target.traitObjects()) {
			const counterSkillId = traitObject.meta.counter_skill;
			const counterSkillCondition = traitObject.meta.counter_skill_condition;

			if (counterSkillId) {
				if (counterSkillId && (!counterSkillCondition || eval(counterSkillCondition))) {
					const counterSkill = $dataSkills[eval(counterSkillId)];
					const action = new Game_Action(target);
					action.setSkill(counterSkill.id);
					const targets = counterSkill.scope == 1 ? [this.subject()] : action.makeTargets();

					for (const counterTarget of targets) {
						const animationId = action.item().animationId;
						if (counterTarget.hp > 0 && ((animationId > 0 && $dataAnimations[animationId].displayType === 0) || animationId === -1)) {
							SceneManager._scene._logWindow.showAnimation(action.subject(), [counterTarget], animationId);
						}
						
						if (CAE?.OnUseEffects) {
							CAE.OnUseEffects.fxCalcDone = false;
						}
						
						for (const effect of action.item().effects) {
							action.applyItemEffect(counterTarget, effect);
						}
						action.applyItemUserEffect(counterTarget);
						SceneManager._scene._logWindow.displayAffectedStatus(counterTarget);
					}
				}
			}
		}
	}
})();
