// Trilobytes - Star Knightess Aura UI Save Menu
// TLB_SKAUISaveMenu.js
//=============================================================================

window.Imported = window.Imported || {};
window.Imported.TLB_SKAUISaveMenu = true;

window.TLB = TLB || {};
TLB.SKAUISaveMenu = TLB.SKAUISaveMenu || {};

/*:
 * @target MZ
 * @plugindesc This plugin modifies the save menu of Star Knightess
 * Aura to reflect the prototypes by Yoroiookami. It is a commissioned work.
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help
 * ============================================================================
 * Introduction
 * ============================================================================
 *
 * This is an ad hoc plugin which modifies the current Scene_Files to match a
 * prototype specified by the client. It will not be compatible with any other
 * project and may not be used by anyone besides the client of the commission.
 *
 * ============================================================================
 * Plugin Parameters
 * ============================================================================
 *
 * ============================================================================
 * Plugin Commands
 * ============================================================================
 *
 * None
 *
 * ============================================================================
 * Compatibility
 * ============================================================================
 *
 * There shouldn't be any compatibility issues with non-menu plugins.
 *
 * ============================================================================
 * Terms of Use
 * ============================================================================
 *
 * Copyright 2022 Auradev
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * ============================================================================
 * 
 * @param difficultyString_cte
 * @text Difficulty String
 * @desc String to use for the Difficulty header.
 * @default DIFFICULTY
 * 
 * @param storyString_cte
 * @text Story String
 * @desc String to use for the Story difficulty.
 * @default Story
 * 
 * @param explorerString_cte
 * @text Explorer String
 * @desc String to use for the Explorer difficulty.
 * @default Explorer
 * 
 * @param normalString_cte
 * @text Normal String
 * @desc String to use for the Normal difficulty.
 * @default Normal
 * 
 * @param hardString_cte
 * @text Hard String
 * @desc String to use for the Hard difficulty.
 * @default Hard
 * 
 * @param nightmareString_cte
 * @text Nightmare String
 * @desc String to use for the Nightmare difficulty.
 * @default Nightmare
 * 
 * @param pageString_cte
 * @text Page String
 * @desc String to use for the "PAGE" text.
 * @default PAGE
 * 
 * @param whichFileString_cte
 * @text Which File String
 * @desc String to use for the which file dialog.
 * @default Which file would you like to 
 * 
 * @param saveString_cte
 * @text Save String
 * @desc String to use for SAVE option.
 * @default SAVE
 * 
 * @param loadString_cte
 * @text Load String
 * @desc String to use for LOAD option.
 * @default LOAD
 * 
 * @param toString_cte
 * @text To String
 * @desc String to use for "to".
 * @default  to
 * 
 * @param fromString_cte
 * @text From String
 * @desc String to use for "from".
 * @default  from
 * 
 * @param autosaveString_cte
 * @text Autosave String
 * @desc String to use for the autosave slot.
 * @default AUTOSAVE
 * 
 * @param timeString_cte
 * @text Time String
 * @desc String to use for the Time section.
 * @default TIME
 * 
 * @param locationString_cte
 * @text Location String
 * @desc String to use for the Location section.
 * @default LOCATION
 * 
 * @param dayString_cte
 * @text Day String
 * @desc String to use for the Day section.
 * @default DAY
 * 
 * @param playtimeString_cte
 * @text Playtime String
 * @desc String to use for the Playtime section.
 * @default PLAYTIME
 * 
 * @param goldString_cte
 * @text Gold String
 * @desc String to use for the Gold section.
 * @default GOLD
 * 
 * @param partyString_cte
 * @text Party String
 * @desc String to use for the Party section.
 * @default PARTY
 *
 * @param difficultyVar
 * @text Difficulty Variable
 * @desc The variable ID that stores the difficulty setting.
 * @type variable
 * @default 624
 *
 * @param listWindow
 * @text List Window
 *
 * @param listWindowX
 * @text X
 * @desc X coordinate of the list window rect.
 * @type number
 * @min -9999
 * @max 9999
 * @default -200
 *
 * @param listWindowY
 * @text Y
 * @desc Y coordinate of the list window rect.
 * @type number
 * @default 70
 *
 * @param listWindowWidth
 * @text Width
 * @desc Width of the list window rect.
 * @type number
 * @default 746
 *
 * @param listWindowHeight
 * @text Height
 * @desc Height of the list window rect.
 * @type number
 * @default 602
 *
 * @param statusWindow
 * @text Status Window
 *
 * @param statusWindowXOffset
 * @text X Offset
 * @desc X offset from the right of the list window.
 * @type number
 * @default 2
 *
 * @param statusWindowWidth
 * @text Width
 * @desc Width of the status window rect.
 * @type number
 * @default 462
 *
 * @param statusWindowHeight
 * @text Height
 * @desc Height of the status window rect.
 * @type number
 * @default 602
 *
 * @param defaultMapName
 * @text Default Map name
 * @desc Default string to use if no map display name is set.
 * @default ???
 *
 * @param autosaveThumbnailDelay
 * @text Autosave Thumbnail Delay
 * @desc Number of ms to wait before taking an autosave screenshot.
 * @type number
 * @default 3000
 *
 * @param savemenu_switch_lefter_arrow_active
 * @text Leftmost Arrow - Active
 * @desc Image to use for leftmost arrow when active.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SWITCH_LEFTER_ARROW_ACTIVE
 *
 * @param savemenu_switch_lefter_arrow_inactive
 * @text Leftmost Arrow - Inactive
 * @desc Image to use for leftmost arrow when inactive.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SWITCH_LEFTER_ARROW_INACTIVE
 *
 * @param savemenu_switch_left_arrow_active
 * @text Left Arrow - Active
 * @desc Image to use for left arrow when active.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SWITCH_LEFT_ARROW_ACTIVE
 *
 * @param savemenu_switch_left_arrow_inactive
 * @text Left Arrow - Inactive
 * @desc Image to use for left arrow when inactive.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SWITCH_LEFT_ARROW_INACTIVE
 *
 * @param savemenu_switch_right_arrow_active
 * @text Right Arrow - Active
 * @desc Image to use for right arrow when active.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SWITCH_RIGHT_ARROW_ACTIVE
 *
 * @param savemenu_switch_right_arrow_inactive
 * @text Right Arrow - Inactive
 * @desc Image to use for right arrow when inactive.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SWITCH_RIGHT_ARROW_INACTIVE
 *
 * @param savemenu_switch_righter_arrow_active
 * @text Rightmost Arrow - Active
 * @desc Image to use for rightmost arrow when active.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SWITCH_RIGHTER_ARROW_ACTIVE
 *
 * @param savemenu_switch_righter_arrow_inactive
 * @text Rightmost Arrow - Inactive
 * @desc Image to use for rightmost arrow when inactive.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SWITCH_RIGHTER_ARROW_INACTIVE
 *
 * @param savemenu_savewindow_bg
 * @text Save Window Background
 * @desc Image to use for save window background.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SAVE_WINDOW_BG
 *
 * @param savemenu_informationwindow_bg
 * @text Information Window Background
 * @desc Image to use for information window background.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_INFORMATION_WINDOW_BG
 *
 * @param savemenu_screenshot_bg
 * @text Screenshot Background
 * @desc Image to use for the snapshot area if no thumbnail is present.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SCREENSHOT_BG
 *
 * @param savemenu_select
 * @text Cursor
 * @desc Image to use for save window cursor.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_SELECT
 *
 * @param savemenu_autosave_select
 * @text Autosave Cursor
 * @desc Image to use for autosave cursor.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_AUTOSAVE_SELECT
 *
 * @param savemenu_corruptionicon
 * @text Corruption Icon
 * @desc Image to use for corruption icon.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_CORRUPTION_ICON
 *
 * @param savemenu_lewdnessicon
 * @text Lewdness Icon
 * @desc Image to use for lewdness icon.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_LEWDNESS_ICON
 *
 * @param savemenu_viceicon
 * @text Vice Icon
 * @desc Image to use for vice icon.
 * @type file
 * @dir img/menu/
 * @default Save_Menu/SAVE_MENU_VICE_ICON
 *
 */

window.parameters = PluginManager.parameters('TLB_SKAUISaveMenu');
TLB.Param = TLB.Param || {};
TLB.Param.SKAUISaM = TLB.Param.SKAUISaM || {};

TLB.SKABase.parseParameters(parameters, TLB.Param.SKAUISaM);

Object.defineProperties(TLB.SKAUISaveMenu, {
	"difficultyString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-difficultyString_cte`) || TLB.Param.SKAUISaM.difficultyString_cte },
	"storyString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-storyString_cte`) || TLB.Param.SKAUISaM.storyString_cte },
	"explorerString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-explorerString_cte`) || TLB.Param.SKAUISaM.explorerString_cte },
	"normalString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-normalString_cte`) || TLB.Param.SKAUISaM.normalString_cte },
	"hardString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-hardString_cte`) || TLB.Param.SKAUISaM.hardString_cte },
	"nightmareString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-nightmareString_cte`) || TLB.Param.SKAUISaM.nightmareString_cte },
	"pageString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-pageString_cte`) || TLB.Param.SKAUISaM.pageString_cte },
	"whichFileString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-whichFileString_cte`) || TLB.Param.SKAUISaM.whichFileString_cte },
	"saveString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-saveString_cte`) || TLB.Param.SKAUISaM.saveString_cte },
	"loadString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-loadString_cte`) || TLB.Param.SKAUISaM.loadString_cte },
	"toString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-toString_cte`) || TLB.Param.SKAUISaM.toString_cte },
	"fromString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-fromString_cte`) || TLB.Param.SKAUISaM.fromString_cte },
	"autosaveString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-autosaveString_cte`) || TLB.Param.SKAUISaM.autosaveString_cte },
	"timeString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-timeString_cte`) || TLB.Param.SKAUISaM.timeString_cte },
	"locationString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-locationString_cte`) || TLB.Param.SKAUISaM.locationString_cte },
	"dayString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-dayString_cte`) || TLB.Param.SKAUISaM.dayString_cte },
	"playtimeString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-playtimeString_cte`) || TLB.Param.SKAUISaM.playtimeString_cte },
	"goldString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-goldString_cte`) || TLB.Param.SKAUISaM.goldString_cte },
	"partyString": { get: () => TextManager.message(`auramzui/TLB_SKAUISaveMenu-partyString_cte`) || TLB.Param.SKAUISaM.partyString_cte }
});

TLB.SKAUISaveMenu.Styles = ((styles) => {
	styles.PAGE_HEADER = new Style({
		textOptions: { bold: true, dropShadow: true, dropShadowX: 1, dropShadowY: 1, shadowOpacity: 0.75 },
		gradient: ["#4a4a60", "#2f2f3e"],
		fontFace: 'franklin-gothic-demi-cond', fontSize: 36
	});
	styles.FOOTER_TEXT = new Style({
		textOptions: { bold: true, italic: true },
		gradient: ["#d9c5de", "#efe6f1", "#d9c5de"],
		fontFace: 'franklin-gothic-demi', fontSize: 18
	});
	styles.SAVE_TITLE = new Style({
		textOptions: { bold: true, outlineGradient: ["#4f4f4f", "#000000"] },
		gradient: ["#e2ba8c", "#b27896"],
		fontFace: 'franklin-gothic-med', fontSize: 24
	}).derive(TLB.SKAUIBase.BASE_DROP_SHADOW);
	styles.SAVE_INFO = new Style({
		textOptions: TLB.SKAUIBase.BASE_DROP_SHADOW,
		gradient: ["#d9c5dd", "#eee5f1", "#d9c4de"],
		fontFace: 'franklin-gothic-med-cond', fontSize: 18,
	});
	styles.INFO_HEADER = styles.FOOTER_TEXT.derive({
		textOptions: { bold: false, italic: false, outlineGradient: ["#4f4f4f", "#000000"] },
		fontFace: 'franklin-gothic-med-cond', fontSize: 26
	});
	styles.INFO_TEXT = styles.FOOTER_TEXT.derive({
		textOptions: { bold: true, italic: false },
		fontFace: 'franklin-gothic-med-cond'
	});
	styles.INFO_LABEL = styles.INFO_TEXT.derive({ gradient: ["#b27996", "#e2bb8c"] });
	styles.VALUE = styles.INFO_TEXT.derive({ fontSize: 14, align: "right" });
	styles.LEVEL = new Style({
		textOptions: { bold: true, outlineThickness: 2 },
		gradient: ["#9bfac1", "#707fba"],
		fontFace: 'franklin-gothic-med-cond', fontSize: 14
	})
	return styles;
})({});

DataManager._snapUrls = {};

DataManager.processThumbnail = function(savefileId) {
	const bitmap = this.makeSavefileBitmap();
	if (bitmap) {
		const snapUrl = bitmap.toDataURL();
		this.saveThumbnail(savefileId, snapUrl);
	}
};

TLB.SKAUISaveMenu.DataManager_saveGame = DataManager.saveGame;
DataManager.saveGame = function(savefileId) {
	const result = TLB.SKAUISaveMenu.DataManager_saveGame.call(this, savefileId);
	if (savefileId > 0) {
		this.processThumbnail(savefileId);
	}
	return result;
};

TLB.SKAUISaveMenu.Scene_Base_onAutosaveSuccess = Scene_Base.prototype.onAutosaveSuccess;
Scene_Base.prototype.onAutosaveSuccess = function() {
	TLB.SKAUISaveMenu.Scene_Base_onAutosaveSuccess.call(this);
	setTimeout(() => {
		DataManager.processThumbnail(0);
	}, TLB.Param.SKAUISaM.autosaveThumbnailDelay);
};

DataManager.getThumbnail = function(savefileId) {
	return this._snapUrls[savefileId];
};

TLB.SKAUISaveMenu.DataManager_makeSavefileInfo = DataManager.makeSavefileInfo;
DataManager.makeSavefileInfo = function() {
	const params = TLB.Param.SKAB;
	const info = TLB.SKAUISaveMenu.DataManager_makeSavefileInfo.call(this);
	info.mapId = $gameMap.mapId();
	info.day = $gameVariables.value(params.dayVar);
	info.corruption = $gameVariables.value(params.currentCorruptionVar);
	info.maxCorruption = $gameVariables.value(params.maxCorruptionVar);
	info.lewdness = $gameVariables.value(params.currentLewdnessVar);
	info.maxLewdness = params.maxLewdness;
	info.vice = $gameVariables.value(params.currentViceVar);
	info.maxVice = params.maxVice;
	if (this._saveName) {
		info.saveName = this._saveName;
		delete this._saveName;
	}

	const savetime = new Date();
	const year = savetime.getFullYear();
	const month = String(savetime.getMonth() + 1).padStart(2, '0');
	const day = String(savetime.getDate()).padStart(2, '0');
	const date = [year, month, day].join('/');
	const hour = String(savetime.getHours()).padStart(2, '0');
	const minute = String(savetime.getMinutes()).padStart(2, '0');
	const time = [hour, minute].join(':');
	const dateTime = date + "  " + time;
	info.savetime = dateTime;

	const difficultyMap = {
		'-2': 'Story',
		'-1': 'Explorer',
		'0': 'Normal',
		'1': 'Hard',
		'2': 'Nightmare'
	};

	info.difficulty = difficultyMap[$gameVariables.value(TLB.Param.SKAUISaM.difficultyVar)];

	info.location = $dataMap.displayName !== "" ? $dataMap.displayName : TLB.Param.SKAUISaM.defaultMapName;
	info.gold = $gameParty.gold();

	info.levels = $gameParty.allMembers().map(member => member.level);

	return info;
};

DataManager.makeSavefileBitmap = function() {
	const bitmap = $gameTemp.getSavefileBitmap();
	if (!bitmap) return null;
	const scale = 26 / 100;
	const newBitmap = new Bitmap(bitmap.width * scale, bitmap.height * scale);
	newBitmap.blt(bitmap, 0, 0, bitmap.width, bitmap.height, 0, 0, newBitmap.width, newBitmap.height);
	return newBitmap;
};

DataManager.makeThumbnailFilename = function(id) {
	return "thumb%1".format(id);
};

DataManager.loadThumbnail = function(id) {
	if (this._snapUrls[id]) {
		return Promise.resolve(this._snapUrls[id]);
	}

	const filename = this.makeThumbnailFilename(id);
	return StorageManager.exists(filename)
		.then(() => StorageManager.loadObject(filename))
		.then(url => {
			this._snapUrls[id] = url;
			return 0;
		});
};

DataManager.saveThumbnail = function(id, url) {
	if (this._snapUrls[id] !== url) {
		this._snapUrls[id] = url;
		const filename = this.makeThumbnailFilename(id);
		StorageManager.saveObject(filename, url);
	}
};

TLB.SKAUISaveMenu.Game_Temp_initialize = Game_Temp.prototype.initialize;
Game_Temp.prototype.initialize = function() {
	TLB.SKAUISaveMenu.Game_Temp_initialize.call(this);
	this._savefileBitmap = null;
};

Game_Temp.prototype.setSavefileBitmap = function(bitmap) {
	this._savefileBitmap = bitmap;
};

Game_Temp.prototype.getSavefileBitmap = function() {
	if (this._savefileBitmap) return this._savefileBitmap;
	else return SceneManager.backgroundBitmap();
};

TLB.SKAUISaveMenu.Game_Interpreter_updateWaitMode = Game_Interpreter.prototype.updateWaitMode;
Game_Interpreter.prototype.updateWaitMode = function() {
	return this._waitMode === "IME Mode" || TLB.SKAUISaveMenu.Game_Interpreter_updateWaitMode.call(this);
}

TLB.SKAUISaveMenu.SceneManager_goto = SceneManager.goto;
SceneManager.goto = function(sceneClass) {
	if (this._scene && this._scene instanceof Scene_Map && sceneClass instanceof Scene_MenuBase) {
		$gameTemp.setSavefileBitmap(SceneManager.snap());
	}
	TLB.SKAUISaveMenu.SceneManager_goto.call(this, sceneClass);
}

Window_SavefileList.prototype.createPageText = function() {
	const textOptions = {
		fill: ['#cdc3d0', '#a79aaa'],
		fontFamily: 'franklin-gothic-demi',
		fontSize: 30,
		stroke: "#000000",
		strokeThickness: 2,
		bold: true
	}
	const pageup = TLB.SKABase.getKey("pageup");
	const pagedown = TLB.SKABase.getKey("pagedown");
	this._puText = new PIXI.Text(String.fromCharCode(pageup), textOptions);
	this._pdText = new PIXI.Text(String.fromCharCode(pagedown), textOptions);
	this._puText.position.x = 574;
	this._puText.position.y = 36;
	this._pdText.position.x = 609;
	this._pdText.position.y = 36;
	this.addChild(this._puText);
	this.addChild(this._pdText);
};

Window_SavefileList.prototype.setFirstButtonFunctions = function(firstButton) {
	const maxPage = this.maxPage();
	firstButton.update = function() {
		Sprite.prototype.update.call(this);
		this.updateFrame();
		this.processTouch();
	};
	firstButton.updateFrame = function() {
		this.bitmap = ImageManager.loadMenu(maxPage > 0 ? TLB.Param.SKAUISaM.savemenu_switch_lefter_arrow_active : TLB.Param.SKAUISaM.savemenu_switch_lefter_arrow_inactive);
		this.opacity = this.isPressed() ? 192 : 255;
	};
	firstButton.onClick = function() {
		this._scrollY = 0;
		this.select(0);
	}.bind(this);
};

Window_SavefileList.prototype.setPrevButtonFunctions = function(prevButton, puText) {
	const maxPage = this.maxPage();
	prevButton.update = function() {
		Sprite.prototype.update.call(this);
		this.updateFrame();
		this.processTouch();
	};
	prevButton.updateFrame = function() {
		this.bitmap = ImageManager.loadMenu(maxPage > 0 ? TLB.Param.SKAUISaM.savemenu_switch_left_arrow_active : TLB.Param.SKAUISaM.savemenu_switch_left_arrow_inactive);
		this.opacity = this.isPressed() ? 192 : 255;
		if (puText) puText.style.fill = maxPage === 0 ? ['#4b4b60', '#2f2f3e'] : ['#bf9e78', '#583346'];
	};
	prevButton.onClick = function() {
		this.cursorPageup();
	}.bind(this);
};

Window_SavefileList.prototype.setNextButtonFunctions = function(nextButton, pdText) {
	const maxPage = this.maxPage();
	nextButton.update = function() {
		Sprite.prototype.update.call(this);
		this.updateFrame();
		this.processTouch();
	};
	nextButton.updateFrame = function() {
		this.bitmap = ImageManager.loadMenu(maxPage > 0 ? TLB.Param.SKAUISaM.savemenu_switch_right_arrow_active : TLB.Param.SKAUISaM.savemenu_switch_right_arrow_inactive);
		this.opacity = this.isPressed() ? 192 : 255;
		if (pdText) pdText.style.fill = maxPage === 0 ? ['#4b4b60', '#2f2f3e'] : ['#bf9e78', '#583346'];
	};
	nextButton.onClick = function() {
		this.cursorPagedown();
	}.bind(this);
};

Window_SavefileList.prototype.setLastButtonFunctions = function(lastButton) {
	const maxPage = this.maxPage();
	lastButton.update = function() {
		Sprite.prototype.update.call(this);
		this.updateFrame();
		this.processTouch();
	};
	lastButton.updateFrame = function() {
		this.bitmap = ImageManager.loadMenu(maxPage > 0 ? TLB.Param.SKAUISaM.savemenu_switch_righter_arrow_active : TLB.Param.SKAUISaM.savemenu_switch_righter_arrow_inactive);
		this.opacity = this.isPressed() ? 192 : 255;
	};
	lastButton.onClick = function() {
		this._scrollY = this.maxPage() * 3 * 132;
		this.select(DataManager.maxSavefiles() - 1);
	}.bind(this);
}

TLB.SKAUISaveMenu.Window_SavefileList_initialize = Window_SavefileList.prototype.initialize;
Window_SavefileList.prototype.initialize = function(rect) {
	TLB.SKAUISaveMenu.Window_SavefileList_initialize.call(this, rect);
	this._canScroll = true;
	this._backgroundSprite = new Sprite();
	let image = TLB.Param.SKAUISaM.savemenu_savewindow_bg;
	const bmp = ImageManager.loadMenu(image);
	this._backgroundSprite.bitmap = bmp;
	this.addChildAt(this._backgroundSprite, 0);

	this._firstButton = new Sprite_Clickable();
	this._firstButton.move(496, 38);
	this._prevButton = new Sprite_Clickable();
	this._prevButton.move(528, 34);
	this._prevButton.width = this._upArrowSprite.width;
	this._prevButton.height = this._upArrowSprite.height;
	this._nextButton = new Sprite_Clickable();
	this._nextButton.move(634, 34);
	this._nextButton.width = this._downArrowSprite.width;
	this._nextButton.height = this._downArrowSprite.height;
	this._lastButton = new Sprite_Clickable();
	this._lastButton.move(687, 38);

	this.addChild(this._firstButton);
	this.addChild(this._prevButton);
	this.addChild(this._nextButton);
	this.addChild(this._lastButton);

	if (!Utils.isMobileDevice()) this.createPageText();

	if (Input.isControllerConnected?.()) {
		this._pdText.alpha = 0;
		this._puText.alpha = 0;
		this._btnUp = new Sprite_ControllerButton("pageup");
		this._btnDown = new Sprite_ControllerButton("pagedown");
		this._btnUp.position.x = 606;
		this._btnUp.position.y = 36;
		this._btnDown.position.x = 631;
		this._btnDown.position.y = 36;
		this._firstButton.hide();
		this._lastButton.hide();
		this._prevButton.x += 32;
		this._nextButton.x += 32;
		this.addChild(this._btnUp);
		this.addChild(this._btnDown);
	}

	this.setFirstButtonFunctions(this._firstButton);
	this.setPrevButtonFunctions(this._prevButton, this._puText);
	this.setNextButtonFunctions(this._nextButton, this._pdText);
	this.setLastButtonFunctions(this._lastButton);

	this.opacity = 0;
	this._contentsSprite.x += 11;
	this._contentsSprite.y += 22;

	this.refresh();
};

Window_SavefileList.prototype.maxPage = function() {
	return Math.ceil(DataManager.maxSavefiles() / 6) - 1;
};

Window_SavefileList.prototype.maxCols = function() {
	return 2;
};

TLB.SKAUISaveMenu.Window_SavefileList_callUpdateHelp = Window_SavefileList.prototype.callUpdateHelp;
Window_SavefileList.prototype.callUpdateHelp = function() {
	TLB.SKAUISaveMenu.Window_SavefileList_callUpdateHelp.call(this);
	if (this.active && this._statusWindow) {
		this._statusWindow.setId(this.indexToSavefileId(this.index()));
	}
};

Window_SavefileList.prototype.indexToSavefileId = function(index) {
	return index + 1;
};

Window_SavefileList.prototype.savefileIdToIndex = function(savefileId) {
	return savefileId - 1;
};

Window_SavefileList.prototype.drawItemBackground = function(index) {
	//
};

Window_SavefileList.prototype.updateArrows = function() {
	this.downArrowVisible = false;
	this.upArrowVisible = false;
};

Window_SavefileList.prototype._refreshArrows = function() {
	//
};

Window_SavefileList.prototype.itemWidth = function() {
	return 346;
};

Window_SavefileList.prototype.itemHeight = function() {
	return 132;
};

Window_SavefileList.prototype.maxVisibleItems = function() {
	return 6;
};

Window_SavefileList.prototype.drawAllItems = function() {
	if (this._characters?.length > 0) {
		for (const slot of this._characters) {
			for (const character of slot) {
				character.destroy();
			}
		}
	}
	this._characters = [[], [], [], [], [], []];
	const page = this.page();
	const maxPage = this.maxPage() + 1;

	const styles = TLB.SKAUISaveMenu.Styles;
	this.drawStyledTextArray([
		[TLB.SKAUISaveMenu.pageString, styles.PAGE_HEADER],
		["•", styles.PAGE_HEADER.derive({ fontFace: 'franklin-gothic-heavy', fontSize: 24 }), { offset: { x: 4, y: 3 }, maxWidth: 40 }],
		[` ${page} `, styles.PAGE_HEADER],
		["/", styles.PAGE_HEADER.derive({ fontFace: 'fuckboi-sans', fontSize: 34 })],
		[`${maxPage}`, styles.PAGE_HEADER],
	], 6, 1, 340);

	Window_Selectable.prototype.drawAllItems.call(this);

	this.drawStyledTextArray([
		[TLB.SKAUISaveMenu.whichFileString, styles.FOOTER_TEXT],
		[this._mode === "save" ? TLB.SKAUISaveMenu.saveString : TLB.SKAUISaveMenu.loadString, styles.FOOTER_TEXT.derive({ gradient: ["#acb2e1", "#616caa"] })],
		[this._mode === "save" ? TLB.SKAUISaveMenu.toString + "?" : TLB.SKAUISaveMenu.fromString + "?", styles.FOOTER_TEXT],
	], 0, 511, 340);

	const rect = new Rectangle(5, 452, 695, 56);
	const info = DataManager.savefileInfo(0);
	this.drawTitle(0, rect.x, rect.y, true);
	this.drawPlaytime(info?.playtime || "00:00:00", rect.x + 607, rect.y);
	this.drawLocation(window.CTE?.getData("map" + info?.mapId || "", "displayName") || info?.location || "<None>", rect.x - 2, rect.y + 27);
};

Window_SavefileList.prototype.drawItem = function(index) {
	const savefileId = this.indexToSavefileId(index);
	const info = DataManager.savefileInfo(savefileId);
	const rect = this.itemRect(index);
	this.resetTextColor();
	this.changePaintOpacity(this.isEnabled(savefileId));
	this.drawTitle(savefileId, rect.x, rect.y, info != null);
	this.drawPlaytime(info?.playtime || "00:00:00", rect.x + 259, rect.y);
	this.drawLocation(window.CTE?.getData("map" + info?.mapId || "", "displayName") || info?.location || "<None>", rect.x, rect.y + 27);
	this.drawParty(index, info?.characters, rect.x + 28, rect.y + 117);
};

Window_SavefileList.prototype.drawTitle = function(savefileId, x, y, slotUsed) {
	const style = (slotUsed || this._mode === "save") ? TLB.SKAUISaveMenu.Styles.SAVE_TITLE : TLB.SKAUISaveMenu.Styles.SAVE_TITLE.derive({ gradient: ['#baac9c', '#7a6b72'] });
	if (savefileId === 0) {
		this.drawStyledText(TextManager.autosave.toUpperCase(), style.derive({ fontFace: 'franklin-gothic-demi' }), x + 9, y + 4, 135);
	} else {
		const info = DataManager.savefileInfo(savefileId);
		const text = info?.saveName ? info.saveName : `${TextManager.file} #` + savefileId;
		this.drawStyledText(text, style, x + 13, y + 4, 220);
	}
};

Window_SavefileList.prototype.drawPlaytime = function(text, x, y) {
	this.drawStyledText(text, TLB.SKAUISaveMenu.Styles.SAVE_INFO.derive({ textOptions: { outlineGradient: ["#4f4f4f", "#000000"] }, fontSize: 20 }), x, y + 3, 135);
};

Window_SavefileList.prototype.drawLocation = function(text, x, y) {
	this.drawStyledText(text, TLB.SKAUISaveMenu.Styles.SAVE_INFO, x + 13, y, 340);
};

Window_SavefileList.prototype.drawParty = function(index, partyData, x, y) {
	if (partyData) {
		let characterX = x;
		let characterY = y;
		for (const data of partyData) {
			const slot = this._characters[Math.floor(index % 6)];
			const tempCharacter = new Game_Character();
			tempCharacter.setImage(data[0], data[1]);
			tempCharacter._screenX = characterX + 24;
			tempCharacter._screenY = characterY + 35;
			tempCharacter.screenX = function() { return this._screenX; }
			tempCharacter.screenY = function() { return this._screenY; }
			const sprite = new Sprite_Character(tempCharacter);
			sprite.move(tempCharacter._screenX, tempCharacter._screenY);
			slot.push(sprite);
			this.addChild(sprite);
			//this.drawCharacter(data[0], data[1], characterX, characterY);
			characterX += 40;
		}
	}
};

Window_SavefileList.prototype.update = function() {
	Window_Selectable.prototype.update.call(this);
	if (this._characters?.length > 0) {
		for (const slot of this._characters) {
			for (const character of slot) {
				character._character.update();
			}
		}
	}
	if (this._index >= 0) {
		for (const character of this._characters[Math.floor(this._index % 6)]) {
			character._character._stepAnime = true;
		}
	}
};

Window_SavefileList.prototype.select = function(index) {
	if (this._characters?.length > 0) {
		if (this._index >= 0) {
			for (const character of this._characters[Math.floor(this._index % 6)]) {
				character._character._stepAnime = false;
			}
		}
	}
	Window_Selectable.prototype.select.call(this, index);
};

Window_SavefileList.prototype.selectAutosave = function() {
	if (this._index >= 0) {
		for (const character of this._characters[Math.floor(this._index % 6)]) {
			character._character._stepAnime = false;
		}
	}
	this._index = -1;
	this._autosaveSelected = true;
	let rect = new Rectangle(12, 477, 695, 56);
	let image = TLB.Param.SKAUISaM.savemenu_autosave_select;
	let bmp = ImageManager.loadMenu(image);
	this._cursorSprite.bitmap = bmp;
	this.setCursorRect(rect.x + 1, rect.y + 1, rect.width, rect.height);
	if (this.active && this._statusWindow) {
		this._statusWindow.setId(this.indexToSavefileId(this.index()));
	}
};

Window_SavefileList.prototype.hitIndex = function() {
	const touchPos = new Point(TouchInput.x, TouchInput.y);
	const localPos = this.worldTransform.applyInverse(touchPos);
	return this.hitTest(localPos.x - 11, localPos.y - 22);
};

Window_SavefileList.prototype.itemRect = function(index) {
	const maxCols = this.maxCols();
	const itemWidth = this.itemWidth();
	const itemHeight = this.itemHeight();
	const colSpacing = this.colSpacing();
	const col = index % maxCols;
	const row = Math.floor(index / maxCols);
	const x = col * itemWidth + (colSpacing * col);
	const y = 51 + row * itemHeight - this.scrollBaseY();
	const width = itemWidth;
	const height = itemHeight;
	return new Rectangle(x, y, width, height);
};

Window_SavefileList.prototype.maxScrollY = function() {
	return this.maxPage() * 3 * 132;
};

Window_SavefileList.prototype.cursorUp = function(wrap) {
	const index = this.index();
	const maxItems = Math.min(this.maxItems(), this.topIndex() + 6);
	const maxCols = this.maxCols();
	if (this.index() === -1) this.smoothSelect(this.topIndex() + 4);
	else if (this.row() === this.topRow()) this.selectAutosave();
	else if (index >= maxCols || (wrap && maxCols === 1)) {
		this.smoothSelect((index - maxCols + maxItems) % maxItems);
	}
};

Window_SavefileList.prototype.cursorDown = function(wrap) {
	const index = this.index();
	const maxItems = Math.min(this.maxItems(), this.topIndex() + 6);
	const maxCols = this.maxCols();
	if (this.row() === this.topRow() + 2) this.selectAutosave();
	else if (index === -1) this.smoothSelect(this.topIndex());
	else if (index < maxItems - maxCols || (wrap && maxCols === 1)) {
		this.smoothSelect((index + maxCols) % maxItems);
	}
};

Window_SavefileList.prototype.cursorLeft = function(wrap) {
	const index = this.index();
	const maxItems = Math.min(this.maxItems(), this.topIndex() + 6);
	const maxCols = this.maxCols();
	const horizontal = this.isHorizontal();
	if (index % 6 === 0) this.selectAutosave();
	else if (index === -1) this.smoothSelect(this.topIndex() + 5);
	else if (maxCols >= 2 && (index > 0 || (wrap && horizontal))) {
		this.smoothSelect((index - 1 + maxItems) % maxItems);
	}
};

Window_SavefileList.prototype.cursorRight = function(wrap) {
	const index = this.index();
	const maxItems = Math.min(this.maxItems(), this.topIndex() + 6);
	const maxCols = this.maxCols();
	const horizontal = this.isHorizontal();
	if (index === this.topIndex() + 5) this.selectAutosave();
	else if (index === -1) this.smoothSelect(this.topIndex());
	else if (maxCols >= 2 && (index < maxItems - 1 || (wrap && horizontal))) {
		this.smoothSelect((index + 1) % maxItems);
	}
};

Window_SavefileList.prototype.cursorPagedown = function() {
	if (this.topIndex() + 6 < this.maxItems()) {
		this._scrollY += 396;
		if (this.index() === -1) this.selectAutosave();
		else this.select(Math.min(this.maxItems() - 1, this.index() + 6));
	} else {
		this._scrollY = 0;
		if (this.index() === -1) this.selectAutosave();
		else this.select(Math.max(0, this.index() - (6 * this.maxPage())));
	}
};

Window_SavefileList.prototype.cursorPageup = function() {
	if (this.topIndex() - 6 >= 0) {
		this._scrollY -= 396;
		if (this.index() === -1) this.selectAutosave();
		else this.select(Math.max(0, this.index() - 6));
	} else {
		this._scrollY = this.maxPage() * 3 * 132;
		if (this.index() === -1) this.selectAutosave();
		else this.select(Math.min(this.maxItems() - 1, 6 * this.maxPage() + this.index()));
	}
};

Window_SavefileList.prototype.processWheelScroll = function() {
	if (this.isWheelScrollEnabled() && this.isTouchedInsideFrame()) {
		const threshold = 20;
		if (TouchInput.wheelY >= threshold) {
			this.cursorPagedown();
		}
		if (TouchInput.wheelY <= -threshold) {
			this.cursorPageup();
		}
	}
};

Window_SavefileList.prototype.updateScrollBase = function(baseX, baseY) {
	const deltaX = baseX - this._scrollBaseX;
	const deltaY = baseY - this._scrollBaseY;
	this._scrollBaseX = baseX;
	this._scrollBaseY = baseY;
	if (this.index() !== -1) {
		this.moveCursorBy(-deltaX, -deltaY);
		this.moveInnerChildrenBy(-deltaX, -deltaY);
	}
};

Window_SavefileList.prototype.checkCoords = function(x, y) {
	const cx = this.origin.x + x - this.padding;
	const cy = this.origin.y + y - this.padding;
	let autosaveRect = new Rectangle(1, 455, 695, 56);
	if (autosaveRect.contains(cx, cy)) {
		this.selectAutosave();
		return -1;
	} else {
		const topIndex = this.topIndex();
		for (let i = 0; i < this.maxVisibleItems(); i++) {
			const index = topIndex + i;
			if (index < this.maxItems()) {
				const rect = this.itemRect(index);
				if (rect.contains(cx, cy)) {
					return index;
				}
			}
		}
	}
};

Window_SavefileList.prototype.hitTest = function(x, y) {
	if (this.innerRect.contains(x, y)) {
		return this.checkCoords(x, y);
	}
	this._autosaveSelected = false;
	return -1;
};

Window_SavefileList.prototype.isTouchOkEnabled = function() {
	return Window_Selectable.prototype.isTouchOkEnabled.call(this) || (this.isOkEnabled() && this._autosaveSelected);
};

Window_SavefileList.prototype.onTouchOk = function() {
	if (this.isTouchOkEnabled() && this._autosaveSelected) this.processOk();
	else Window_Selectable.prototype.onTouchOk.call(this);
};

Window_SavefileList.prototype.colSpacing = function() {
	return 6;
};

Window_SavefileList.prototype.rowSpacing = function() {
	return 0;
};

Window_SavefileList.prototype.refreshCursor = function() {
	if (this.index() >= 0) {
		const rect = this.itemRect(this.index());
		rect.x += 11;
		rect.y += 22;
		let image = TLB.Param.SKAUISaM.savemenu_select;
		let bmp = ImageManager.loadMenu(image);
		this._cursorSprite.bitmap = bmp;
		this.setCursorRect(rect.x + 1, rect.y + 1, rect.width, rect.height);
		this.cursorVisible = true;
	} else {
		let image = TLB.Param.SKAUISaM.savemenu_autosave_select;
		let bmp = ImageManager.loadMenu(image);
		this._cursorSprite.bitmap = bmp;
		this.setCursorRect(12, 477, 695, 56);
		this.cursorVisible = false;
	}
};

Window_SavefileList.prototype._createCursorSprite = function() {
	this._cursorSprite = new Sprite();
	let image = TLB.Param.SKAUISaM.savemenu_select;
	let bmp = ImageManager.loadMenu(image);
	this._cursorSprite.bitmap = bmp;
	this._clientArea.addChild(this._cursorSprite);
};

Window_SavefileList.prototype._refreshCursor = function() {
	//
};

Window_SavefileList.prototype.page = function() {
	return this.topIndex() / this.maxVisibleItems() + 1;
};

TLB.SKAUISaveMenu.Window_SavefileList_selectSavefile = Window_SavefileList.prototype.selectSavefile;
Window_SavefileList.prototype.selectSavefile = function(savefileId) {
	TLB.SKAUISaveMenu.Window_SavefileList_selectSavefile.call(this, savefileId);
	this._scrollY = Math.floor(this.index() / 6) * 3 * 132;
	this.updateScrollBase(this._scrollX, this._scrollY);
};

Window_SavefileList.prototype.onTouchScroll = function() {
	if (this._canScroll) {
		this._canScroll = false;
		const accelX = this._scrollLastTouchX - TouchInput.x;
		const accelY = this._scrollLastTouchY - TouchInput.y;
		if (accelX < -1 || accelY < -1) this.cursorPageup();
		if (accelX > 1 || accelY > 1) this.cursorPagedown();
	}
};

Window_SavefileList.prototype.onTouchScrollEnd = function() {
	this._scrollTouching = false;
	this._canScroll = true;
	this.cursorVisible = this._scrollLastCursorVisible;
};

TLB.SKAUISaveMenu.Scene_File_create = Scene_File.prototype.create;
Scene_File.prototype.create = function() {
	TLB.SKAUISaveMenu.Scene_File_create.call(this);
	this.createStatusWindow();
	this._listWindow._statusWindow = this._statusWindow;
	this._helpWindow.hide();
};

TLB.SKAUISaveMenu.Scene_File_start = Scene_File.prototype.start;
Scene_File.prototype.start = function() {
	TLB.SKAUISaveMenu.Scene_File_start.call(this);
	this._listWindow.callUpdateHelp();
};

Scene_File.prototype.listWindowRect = function() {
	const params = TLB.Param.SKAUISaM;
	const wx = params.listWindowX;
	const wy = params.listWindowY;
	const ww = params.listWindowWidth;
	const wh = params.listWindowHeight;
	return new Rectangle(wx, wy, ww, wh);
};

Scene_File.prototype.createStatusWindow = function() {
	const rect = this.statusWindowRect();
	this._statusWindow = new Window_SavefileStatus(rect);
	this.addWindow(this._statusWindow);
};

Scene_File.prototype.statusWindowRect = function() {
	const params = TLB.Param.SKAUISaM;
	const wx = this._listWindow.x + this._listWindow.width + params.statusWindowXOffset;
	const wy = this._listWindow.y;
	const ww = params.statusWindowWidth;
	const wh = params.statusWindowHeight;
	return new Rectangle(wx, wy, ww, wh);
};

Scene_File.prototype.onControllerDisconnect = function() {
	this._listWindow._btnUp.hide();
	this._listWindow._btnDown.hide();
	if (this._listWindow._pdText) {
		this._listWindow._pdText.alpha = 1;
		this._listWindow._puText.alpha = 1;
	}
	this._listWindow._firstButton.show();
	this._listWindow._lastButton.show();
	this._listWindow._prevButton.x = 528;
	this._listWindow._nextButton.x = 634;
};

Scene_File.prototype.onControllerConnect = function() {
	if (this._listWindow._btnUp) {
		this._listWindow._btnUp.show();
		this._listWindow._btnDown.show();
	} else {
		this._listWindow._btnUp = new Sprite_ControllerButton("pageup");
		this._listWindow._btnDown = new Sprite_ControllerButton("pagedown");
		this._listWindow._btnUp.position.x = 606;
		this._listWindow._btnUp.position.y = 36;
		this._listWindow._btnDown.position.x = 631;
		this._listWindow._btnDown.position.y = 36;
		this._listWindow.addChild(this._listWindow._btnUp);
		this._listWindow.addChild(this._listWindow._btnDown);
	}
	if (this._listWindow._pdText) {
		this._listWindow._puText.alpha = 0;
		this._listWindow._pdText.alpha = 0;
	}
	this._listWindow._firstButton.hide();
	this._listWindow._lastButton.hide();
	this._listWindow._prevButton.x = 560;
	this._listWindow._nextButton.x = 666;
};

class Window_SavefileStatus extends Window_Base {
	constructor(rect) {
		super(rect);
		this._backgroundSprite = new Sprite();
		let image = TLB.Param.SKAUISaM.savemenu_informationwindow_bg;
		let bmp = ImageManager.loadMenu(image);
		this._backgroundSprite.bitmap = bmp;
		this.addChildAt(this._backgroundSprite, 0);
		const thumbnailBg = new Sprite();
		image = TLB.Param.SKAUISaM.savemenu_screenshot_bg;
		bmp = ImageManager.loadMenu(image);
		thumbnailBg.bitmap = bmp;
		thumbnailBg.move(23, 61);
		this.addChildAt(thumbnailBg, 1);
		this.opacity = 0;
		this.setId(1);
	}

	setId(id) {
		const oldId = this._id;
		this._id = id;
		if (oldId != id) {
			this.refresh();
		}
	}

	refresh() {
		const id = this._id;
		DataManager.savefileExists(id).then(() => this.drawSaveInfo(true), () => this.drawSaveInfo(false));
	}

	drawSaveInfo(valid) {
		this.contents.clear();
		const id = this._id;
		const info = DataManager.savefileInfo(id);
		if (info) {
			this.loadSnapshot(id, valid);
			if (info.corruption) this.drawCorruption(info, valid);
			if (info.lewdness) this.drawLewdness(info, valid);
			if (info.vice) this.drawVice(info, valid);
			this.drawSaveTime(info, valid);
			this.drawDifficulty(info, valid);
			this.drawLocation(info, valid);
			this.drawPlayTime(info, valid);
			if (info.gold != null) this.drawGold(info, valid);
			this.drawParty(info, valid);
		}

		const text = id === 0 ? TLB.SKAUISaveMenu.autosaveString : info?.saveName ? info.saveName : `${TLB.SKAUISaveMenu.saveString} #` + id;
		this.drawStyledText(text, TLB.SKAUISaveMenu.Styles.INFO_HEADER, 17, 10, 220);
	}

	loadSnapshot(savefileId, valid) {
		DataManager.loadThumbnail(savefileId)
			.then(() => this.drawSnapshot(savefileId, valid))
			.catch(() => 0);
	}

	drawSnapshot(savefileId, valid) {
		const snapUrl = DataManager.getThumbnail(savefileId);
		if (valid && snapUrl) {
			const hasEncryptedImages = Utils._hasEncryptedImages;
			Utils._hasEncryptedImages = false;
			const bitmap = ImageManager.loadBitmapFromUrl(snapUrl);
			Utils._hasEncryptedImages = hasEncryptedImages;
			this.changePaintOpacity(true);
			bitmap.addLoadListener(() => {
				this.contents.blt(bitmap, 0, 0, bitmap.width, bitmap.height, 13, 51);
			});
		}
	}

	drawValue(x, y, current, max) {
		const styleSeparator = TLB.SKAUISaveMenu.Styles.VALUE.derive({ fontFace: 'fuckboi-sans', fontSize: 13 });
		styleSeparator.derive({ fontSize: 5 }).applyFontSetting(this.contents);
		const offset1 = this.textWidth(" ");
		TLB.SKAUISaveMenu.Styles.VALUE.derive({ fontSize: 10 }).applyFontSetting(this.contents);
		const offset2 = offset1 + this.textWidth(" ");

		this.drawStyledTextArray([
			[max, TLB.SKAUISaveMenu.Styles.VALUE],
			["/", styleSeparator, { offset: { x: -offset1 } }],
			[current, TLB.SKAUISaveMenu.Styles.VALUE, { offset: { x: -offset2 } }],
		], x, y, 50);
	}

	drawCorruption(info, valid) {
		const bitmap = ImageManager.loadMenu(TLB.Param.SKAUISaM.savemenu_corruptionicon);
		bitmap.addLoadListener(() => {
			this.contents.blt(bitmap, 0, 0, bitmap.width, bitmap.height, 365, 68);
		});
		if (valid && info.corruption != null) {
			this.drawValue(358, 42, info.corruption, info.maxCorruption);
		}
	}

	drawLewdness(info, valid) {
		const bitmap = ImageManager.loadMenu(TLB.Param.SKAUISaM.savemenu_lewdnessicon);
		bitmap.addLoadListener(() => {
			this.contents.blt(bitmap, 0, 0, bitmap.width, bitmap.height, 369, 138);
		});
		if (valid && info.lewdness != null) {
			this.drawValue(357, 112, info.lewdness, info.maxLewdness);
		}
	}

	drawVice(info, valid) {
		const bitmap = ImageManager.loadMenu(TLB.Param.SKAUISaM.savemenu_viceicon);
		bitmap.addLoadListener(() => {
			this.contents.blt(bitmap, 0, 0, bitmap.width, bitmap.height, 369, 201);
		});
		if (valid && info.vice != null) {
			this.drawValue(357, 175, info.vice, info.maxVice);
		}
	}

	drawSaveTime(info, valid) {
		if (valid && info.savetime) {
			const x = 18;
			const y = 241;
			this.drawStyledText(TLB.SKAUISaveMenu.timeString, TLB.SKAUISaveMenu.Styles.INFO_LABEL, x, y, 40);
			this.drawStyledText(info.savetime, TLB.SKAUISaveMenu.Styles.INFO_TEXT, x + 20, y + 21, 160);
		}
	}

	drawDifficulty(info, valid) {
		if (valid && info.difficulty) {
			const x = 222;
			const y = 241;
			this.drawStyledText(TLB.SKAUISaveMenu.difficultyString, TLB.SKAUISaveMenu.Styles.INFO_LABEL, x, y, 80);
			const text = TLB.SKAUISaveMenu[`${info.difficulty.toLowerCase()}String`];
			this.drawStyledText(text, TLB.SKAUISaveMenu.Styles.INFO_TEXT, x + 20, y + 21, 160);
		}
	}

	drawLocation(info, valid) {
		if (valid && info.location) {
			const x = 18;
			const y = 302;
			this.drawStyledText(TLB.SKAUISaveMenu.locationString, TLB.SKAUISaveMenu.Styles.INFO_LABEL, x, y, 80);
			this.drawStyledText(
				window.CTE?.getData("map" + info.mapId || "", "displayName") || info.location,
				TLB.SKAUISaveMenu.Styles.INFO_TEXT, x + 20, y + 21, 300
			);
		}
	}

	drawPlayTime(info, valid) {
		if (valid && info.playtime) {
			const x = 18;
			const y = 364;
			const text = info.day ? `${TLB.SKAUISaveMenu.dayString} ${info.day}   ${info.playtime}` : info.playtime;
			this.drawStyledText(TLB.SKAUISaveMenu.playtimeString, TLB.SKAUISaveMenu.Styles.INFO_LABEL, x, y, 70);
			this.drawStyledText(text, TLB.SKAUISaveMenu.Styles.INFO_TEXT, x + 20, y + 21, info.day ? 130 : 70);
		}
	}

	drawGold(info, valid) {
		if (valid && typeof info.gold === 'number') {
			const x = 222;
			const y = 364;
			this.drawStyledText(TLB.SKAUISaveMenu.goldString, TLB.SKAUISaveMenu.Styles.INFO_LABEL, x, y, 70);
			this.drawStyledText(TLB.SKABase.goldAsString(info.gold), TLB.SKAUISaveMenu.Styles.INFO_TEXT, x + 20, y + 21, 70);
		}
	}

	drawParty(info, valid) {
		if (valid && info.characters) {
			const x = 18;
			const y = 424;
			this.drawStyledText(TLB.SKAUISaveMenu.partyString, TLB.SKAUISaveMenu.Styles.INFO_LABEL, x, y, 70);
			const levelTextStyle = TLB.SKAUISaveMenu.Styles.INFO_TEXT.derive({ fontSize: 14 });

			let characterX = 33;
			let characterY = 506;
			let index = 0;
			for (const data of info.characters) {
				this.drawCharacter(data[0], data[1], characterX, characterY);
				this.drawStyledText(TextManager.levelA, TLB.SKAUISaveMenu.Styles.LEVEL, characterX + 20, characterY - 41, 70);
				const level = info.levels?.[index] || "?";
				this.drawStyledText(level, levelTextStyle, characterX + 20, characterY - 26);
				index++;
				if (index === 5) {
					characterX = 33;
					characterY += 59;
				} else {
					characterX += 84;
				}
			}
			this.resetFontSettings();
		}
	}
}