//=============================================================================
// RPG Maker MZ - Demo Deploy
// ----------------------------------------------------------------------------
// (C)2025 aura-dev
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// 1.0.0 2025/03/27
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Demo Deploy
 * @author aura-dev
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help demo.js
 *
 * Demo deploy specific customizations.
 *
 * @param demoMode
 * @type boolean
 * @text Demo Mode
 * @desc Whether the game is in demo mode or not
 * @default false
 * 
 * @param demoSwitch
 * @type switch
 * @text Demo Switch
 * @desc Will be set to on when starting a plugin in demo mode
 * @default 6
 * 
 * @param demoText_cte
 * @type text
 * @text Demo Text
 * @desc Will be appended to the game version if demo is active
 * @default -demo
 *
 * @param maxDays
 * @type number
 * @text Max Days
 * @desc The maximum number of days that can be played in the demo
 * @default 40
 */

window.AuraMZ = window.AuraMZ || {};
AuraMZ.Demo = {};

(($) => {
	const PLUGIN_ID = "ska_demo";
	const PARAMS = PluginManager.parameters(PLUGIN_ID);

	Object.defineProperties($, {
		"demoText": { get: () => TextManager.message(`auramz/${PLUGIN_ID}-demoText_cte`) || PARAMS.demoText_cte }
	});

	$.demoMode = PARAMS["demoMode"] === 'true';
	$.demoSwitch = parseInt(PARAMS["demoSwitch"]);
	$.maxDays = parseInt(PARAMS["maxDays"])

	if ($.demoMode) {
		if (AuraMZ.gameVersion) {
			AuraMZ.gameVersion = AuraMZ.gameVersion + $.demoText;
		}

		$._Scene_Title_start = Scene_Title.prototype.start;
		Scene_Title.prototype.start = function() {
			$._Scene_Title_start.call(this);
			if (AuraMZ.gameVersion) {
				AuraMZ.gameVersion = AuraMZ.gameVersion + $.deluxeText;
			}
		};

		const _DataManager_createGameObjects = DataManager.createGameObjects;
		DataManager.createGameObjects = function() {
			_DataManager_createGameObjects.call(this);
			// Turn on demo flag
			$gameSwitches.setValue($.demoSwitch, true);
		};
	}
})(AuraMZ.Demo);