//=============================================================================
// RPG Maker MZ - Liora Calendar
// ----------------------------------------------------------------------------
// (C)2025 aura-dev
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// 1.0.0 02/04/2025
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Liora Calendar System
 * @author aura-dev
 * @url https://gitgud.io/auragamedev/auramz
 *
 * @help liora_calendar.js
 *
 * Implements the calendar system
 * 
 * @param weekdays
 * @type struct<weekday>[]
 * @text Weekdays
 * @desc The list of weekdays
 * @default []
 */
/*~struct~weekday:
*
* @param name_cte
* @text Name
* @type text
* @desc Name of the weekday
*
* @param elementId
* @text Element ID
* @type number
* @desc The associated element
*
*/

window.Liora = window.Liora || {};
Liora.Calendar = {};

(($) => {
	const PLUGIN_ID = "liora_calendar";
	$.Params = PluginManager.parameters(PLUGIN_ID);
	$.weekdays = [];
	$.elementBoost = 1.5;

	// Load the weekday list
	$._Scene_Boot_start = Scene_Boot.prototype.start;
	Scene_Boot.prototype.start = function() {
		$._Scene_Boot_start.call(this);
		const paramWeekdays = JSON.parse($.Params["weekdays"]);
		for (let i = 0; i < paramWeekdays.length; i++) {
			const jsonWeekday = paramWeekdays[i];
			const weekday = JSON.parse(jsonWeekday);
			weekday.elementId = JSON.parse(weekday.elementId);
			$.weekdays.push(weekday);
		}
	};

	// Get the current weekday ID
	$.currentWeekdayId = function() {
		return ($gameVariables.value(2) - 1) % $.weekdays.length;
	};
	
	// Gets the name of the given weekday
	$.weekday = function(weekdayId) {
		return $.weekdays[weekdayId];
	}
	
	// Gets the icon index of the given weekday
	$.weekdayIconIndex = function(weekday) {
		const elementId = weekday.elementId;
		return eval(AuraMZ.BattleUX.mapElementIdToIconId);
	}
	
	// Gets the name of the current weekday
	$.currentWeekday = function() {
		return $.weekday($.currentWeekdayId());
	};
	
	// Inject drawing of current weekday in main menu
	$._Scene_Menu_start = Scene_Menu.prototype.start;
	Scene_Menu.prototype.start = function() {
		$._Scene_Menu_start.call(this);
		this.createWeekdayDisplay();
	};
	
	// Draws the current weekday with the element icon
	Scene_Menu.prototype.createWeekdayDisplay = function() {
		const params = TLB.Param.SKAUIM;
		const xPos = params.mainmenu_corruptiongauge_xoffset;
		const yPos = params.mainmenu_corruptiongauge_yoffset;
		const labelX = xPos + params.mainmenu_corruptiongauge_label_xoffset;
		const labelY = yPos + params.mainmenu_corruptiongauge_label_yoffset;
		const fontSize = params.mainmenu_day_fontsize;
		const gradient = params.mainmenu_corruptiongauge_label_gradient;
		const outlineGradient = TLB.Param.SKAUIB.textoutlinegradient;
		const bmp = this._drawingLayer.bitmap;
		bmp.fontFace = 'franklin-gothic-med-cond';
		bmp.fontSize = fontSize;
		const textOptions = {
			bitmap: bmp,
			outlineThickness: 3,
			outlineGradient: outlineGradient,
			dropShadow: true,
			dropShadowX: 0,
			dropShadowY: 2,
			shadowOpacity: 0.75
		};
		bmp.letterSpacing = 1;
		const winClass = Window_Base.prototype;
		const weekday = $.currentWeekday();
		const name = TextManager.message(`${PLUGIN_ID}-weekdays-${$.currentWeekdayId()}-name_cte`) || weekday.name_cte;
		winClass.drawGradientText.call(this._statusWindow, name.toUpperCase(), gradient, labelX, labelY, 110, "left", { ...textOptions });
		const elementIconId = $.weekdayIconIndex(weekday);
		winClass.drawIcon.call(this._statusWindow, elementIconId, labelX * 0.6675, labelY * 0.855);
		bmp.resetLetterSpacing();
	};
	
	// Boost damage if the item element and the weekday element align
	$._Game_Action_makeDamageValue = Game_Action.prototype.makeDamageValue;
	Game_Action.prototype.makeDamageValue = function(target, critical) {
		let value = $._Game_Action_makeDamageValue.call(this, target, critical);
		const weekday = $.currentWeekday();
		const boostedElement = weekday.elementId;
		if (boostedElement == this.item().damage.elementId) {
			value = Math.ceil(value * $.elementBoost);
		}
		return value;
	};

})(Liora.Calendar);

