//=============================================================================
// ItemNameEscape.js
// ----------------------------------------------------------------------------
// (C)2015 Triacontane
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// 1.1.2 2022/01/07 戦闘中に変更した変数がその戦闘中に開いたスキルやアイテム一覧に反映されない問題を修正
//                  パフォーマンス対策
// 1.1.1 2021/11/22 既存データをロードした直後だと、何らかの変数操作が実行されるまで変数値の制御文字が反映されない問題を修正
// 1.1.0 2021/11/22 MZで動作するよう修正
// 1.0.1 2015/12/24 マップデータが歯抜けになっている場合に発生するエラーを対応
// 1.0.0 2015/12/20 初版
// ----------------------------------------------------------------------------
// [Blog]   : https://triacontane.blogspot.jp/
// [Twitter]: https://twitter.com/triacontane/
// [GitHub] : https://github.com/triacontane/
//=============================================================================

/*:
 * @plugindesc 項目名の制御文字適用プラグイン
 * @target MZ
 * @url https://github.com/triacontane/RPGMakerMV/tree/mz_master/ItemNameEscape.js
 * @base PluginCommonBase
 * @orderAfter PluginCommonBase
 * @author トリアコンタン
 *
 * @help ItemNameEscape.js
 *
 * データベースの項目に制御文字が使えるようになります。
 * 利用可能な制御文字は以下の通りです。
 * 　\V, \N, \P, \G
 *
 * 対象項目は項目は以下の通りです。
 * 　「名称」「説明文」
 *
 * このプラグインの利用にはベースプラグイン『PluginCommonBase.js』が必要です。
 * 『PluginCommonBase.js』は、RPGツクールMZのインストールフォルダ配下の
 * 以下のフォルダに格納されています。
 * dlc/BasicResources/plugins/official
 *
 * 利用規約：
 *  作者に無断で改変、再配布が可能で、利用形態（商用、18禁利用等）
 *  についても制限はありません。
 *  このプラグインはもうあなたのものです。
 */
(() => {
	'use strict';

	//=============================================================================
	// DataManager
	//  データベースの「名称」と「説明文」に制御文字を適用します。
	//=============================================================================
	const _DataManager_onLoad = DataManager.onLoad;
	DataManager.onLoad = function(object) {
		_DataManager_onLoad.apply(this, arguments);
		if (Array.isArray(object) && object !== $dataMapInfos) {
			for (let i = 1; i < object.length; i++) {
				const data = object[i];
				if (data != null && data.name?.match(/\\/g)) {
					const originalGetter = Object.getOwnPropertyDescriptor(data, "name");
					Object.defineProperty(data, "name", {
						get: function() {
							const original = originalGetter.get ? originalGetter.get.call(data) : originalGetter.value;
							// Don't perform substitutions when the game hasn't started as variables and switches aren't available
							return $gameMap?.mapId() != 0 ? DataManager.convertName(original) : original;
						},
						configurable: true
					});
				}
			}
		}
	};

	DataManager.convertName = function(name) {
		return PluginManagerEx.convertEscapeCharacters(name);
	};
})();

