/** ============================================================================
 *
 *  Luna_Quit2Desktop.js
 * 
 *  Build Date: 9/29/2020
 * 
 *  Made with LunaTea -- Haxe
 *
 * =============================================================================
*/
// Generated by Haxe 4.1.3
/*:
* @plugindesc Adds options to quit directly to desktop in the title or game exit scenes.
* <Luna_Quit2Desktop>
*
* @target MZ MV
* @author LunaTechs | inc0der
* @url https://lunatechs.dev/plugins/luna-quit2desktop/
* 
* @param showOnGameEnd
* @text Show on GameEnd Scene
* @type boolean
* @desc Do you want a quit command on the GameEnd scene?
* @default true
* @on Show
* @off Hide
*
* @param questionText_cte
* @text Game End Prompt Text
* @type text
* @desc The prompt text that appears on the GameEnd scene.
* @default Are you sure you want to exit to desktop?
* 
* @param gameEndText_cte
* @text Game End Command Text
* @type text
* @desc The command text that appears on the GameEnd scene.
* @default To Desktop
*
* @param yesText_cte
* @text Yes Text
* @type text
* @desc The string to use for "Yes".
* @default Yes
*
* @param noText_cte
* @text No Text
* @type text
* @desc The string to use for "No".
* @default No
* 
* @param gameEndPosition
* @text Game End Command Position
* @type number
* @desc The position of the quit command on the GameEnd scene.
* @default 2

@help
This plugin allows you to add command to quit to the desktop on the title scene
or on the game end scene.

MIT License
Copyright (c) 2020 LunaTechsDev
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE
* 
*/

window.ThirdParty = window.ThirdParty || {};
ThirdParty.Quit = {};

(function($hx_exports, $global) {
  "use strict";
  const PLUGIN_NAME = "Luna_Quit2Desktop";
  const PARAMS = PluginManager.parameters(PLUGIN_NAME);
  
  Object.defineProperties(ThirdParty.Quit, {
    "questionText": { get: () => TextManager.message(`thirdparty/${PLUGIN_NAME}-questionText_cte`) || PARAMS.questionText_cte },
    "gameEndText": { get: () => TextManager.message(`thirdparty/${PLUGIN_NAME}-gameEndText_cte`) || PARAMS.gameEndText_cte },
    "yesText": { get: () => TextManager.message(`thirdparty/${PLUGIN_NAME}-yesText_cte`) || PARAMS.yesText_cte },
    "noText": { get: () => TextManager.message(`thirdparty/${PLUGIN_NAME}-noText_cte`) || PARAMS.noText_cte }
	});
  
	class Main {
		static main() {
			let rawParams = PluginManager.parameters(PLUGIN_NAME);
			Main.Params = utils_Parse.parseParameters(rawParams);

			//=============================================================================
			// Scene_GameEnd
			//=============================================================================
			let _Scene_GameEnd_create = Scene_GameEnd.prototype.create;
			Scene_GameEnd.prototype.create = function() {
				_Scene_GameEnd_create.call(this);
				this.createConfirmWindow();
				this.createHelpWindow();
				this._helpWindow.hide();
				this._helpWindow.close();
			};
			let _Scene_GameEnd_createCommandWindow =
				Scene_GameEnd.prototype.createCommandWindow;
			Scene_GameEnd.prototype.createCommandWindow = function() {
				_Scene_GameEnd_createCommandWindow.call(this);
				this._commandWindow.setHandler(
					"toDesktop",
					$bind(this, this.quit2desktop)
				);
			};
			let _Scene_GameEnd_createConfirmWindow =
				Scene_GameEnd.prototype.createConfirmWindow;
			Scene_GameEnd.prototype.createConfirmWindow = function() {
				let rect = new Rectangle(
					this._commandWindow.x,
					this._commandWindow.y,
					250,
					this.calcWindowHeight(2, true)
				);
				let confirmWindow = new Window_$Confirm(rect);
				this._confirmWindow = confirmWindow;
				confirmWindow.setHandler("ok", $bind(this, this.onConfirm));
				confirmWindow.close();
				this.addWindow(confirmWindow);
			};
			let _Scene_GameEnd_commandWindowRect = Scene_GameEnd.prototype.commandWindowRect;
			Scene_GameEnd.prototype.commandWindowRect = function() {
				const rect = _Scene_GameEnd_commandWindowRect.call(this);
				if (Utils.isNwjs()) {
					rect.height = this.calcWindowHeight(3, true);
				} else {
					rect.height = this.calcWindowHeight(2, true);
				}
				return rect;
			};
			let _Scene_GameEnd_quit2desktop = Scene_GameEnd.prototype.quit2desktop;
			Scene_GameEnd.prototype.quit2desktop = function() {
				this._commandWindow.close();
				this._helpWindow.show();
				this._helpWindow.open();
				this._helpWindow.setText(ThirdParty.Quit.questionText);
				this._confirmWindow.open();
				this._confirmWindow.activate();
			};
			let _Scene_GameEnd_onConfirm = Scene_GameEnd.prototype.onConfirm;
			Scene_GameEnd.prototype.onConfirm = function() {
				let confirmWindow = this._confirmWindow;
				let choice = confirmWindow.currentSymbol();
				confirmWindow.close();
				if (choice == "yes") {
					SceneManager.exit();
				}
				this._helpWindow.close();
				this._commandWindow.open();
				this._commandWindow.activate();
			};

			//=============================================================================
			// Window_GameEnd
			//=============================================================================
			let _Window_GameEnd_makeCommandList =
				Window_GameEnd.prototype.makeCommandList;
			Window_GameEnd.prototype.makeCommandList = function() {
				_Window_GameEnd_makeCommandList.call(this);
				if (Utils.isNwjs()) {
					let index = Math.round(Main.Params.gameEndPosition - 1);
					this._list.splice(index, 0, {
						name: ThirdParty.Quit.gameEndText,
						symbol: "toDesktop",
						enabled: true,
						ext: null,
					});
				}
			};
			let _Window_GameEnd_maxItems = Window_GameEnd.prototype.maxItems;
			Window_GameEnd.prototype.maxItems = function() {
				return Window_Command.prototype.maxItems.call(this);
			};
		}
	}

	class haxe_iterators_ArrayIterator {
		constructor(array) {
			this.current = 0;
			this.array = array;
		}
		hasNext() {
			return this.current < this.array.length;
		}
		next() {
			return this.array[this.current++];
		}
	}

	class scenes_Scene_$GameEnd extends Scene_GameEnd {
		constructor() {
			super();
		}
		createCommandWindow() {
			_Scene_GameEnd_createCommandWindow.call(this);
			this._commandWindow.setHandler(
				"toDesktop",
				$bind(this, this.quit2desktop)
			);
		}
		createConfirmWindow() {
			let rect = new Rectangle(
				this._commandWindow.x,
				this._commandWindow.y,
				250,
				this.calcWindowHeight(2, true)
			);
			let confirmWindow = new Window_$Confirm(rect);
			this._confirmWindow = confirmWindow;
			confirmWindow.setHandler("ok", $bind(this, this.onConfirm));
			confirmWindow.close();
			this.addWindow(confirmWindow);
		}
		quit2desktop() {
			this._commandWindow.close();
			this._helpWindow.show();
			this._helpWindow.open();
			this._helpWindow.setText(ThirdParty.Quit.questionText);
			this._confirmWindow.open();
			this._confirmWindow.activate();
		}
		onConfirm() {
			let confirmWindow = this._confirmWindow;
			let choice = confirmWindow.currentSymbol();
			confirmWindow.close();
			if (choice == "yes") {
				SceneManager.exit();
			}
			this._helpWindow.close();
			this._commandWindow.open();
			this._commandWindow.activate();
		}
	}

	class utils_Fn {
		static proto(obj) {
			return obj.prototype;
		}
		static updateProto(obj, fn) {
			return fn(obj.prototype);
		}
		static updateEntity(obj, fn) {
			return fn(obj);
		}
	}

	class utils_Parse {
		static parseParameters(parameters) {
			return (function(string) {
				function superParse(string) {
					var temp;
					try {
						temp = JsonEx.parse(
							typeof string === "object" ? JsonEx.stringify(string) : string
						);
					} catch (e) {
						return string;
					}
					if (typeof temp === "object") {
						Object.keys(temp).forEach(function(key) {
							temp[key] = superParse(temp[key]);
							if (temp[key] === " ") {
								temp[key] = null;
							}
						});
					}
					return temp;
				}
				return superParse(JsonEx.stringify(string));
			})(parameters);
		}
	}

	class Window_$Confirm extends Window_Command {
		constructor(rect) {
			super(rect);
		}
		makeCommandList() {
			this.addCommand(ThirdParty.Quit.yesText, "yes", true);
			this.addCommand(ThirdParty.Quit.noText, "no", true);
		}
	}

	class windows_Window_$GameEnd extends Window_GameEnd {
		constructor() {
			super();
		}
		maxItems() {
			return Window_Command.prototype.maxItems.call(this);
		}
	}

	var $_;
	function $bind(o, m) {
		if (m == null) return null;
		if (m.__id__ == null) m.__id__ = $global.$haxeUID++;
		var f;
		if (o.hx__closures__ == null) o.hx__closures__ = {};
		else f = o.hx__closures__[m.__id__];
		if (f == null) {
			f = m.bind(o);
			o.hx__closures__[m.__id__] = f;
		}
		return f;
	}
	$global.$haxeUID |= 0;
	Main.main();
})(
	typeof exports != "undefined"
		? exports
		: typeof window != "undefined"
			? window
			: typeof self != "undefined"
				? self
				: this,
	typeof window != "undefined"
		? window
		: typeof global != "undefined"
			? global
			: typeof self != "undefined"
				? self
				: this
);
