import json
import os
import shutil
import sys
from utils.config import DEFAULT_LANGUAGE
from utils.json_helper import clear_values

"""Copies a language folder but sets all translation values to empty strings."""
def clear_translation_keys(dest_folder):
    if os.path.exists(dest_folder):
        print(f"Error: Destination folder '{dest_folder}' already exists.")
        return
    
    shutil.copytree(DEFAULT_LANGUAGE, dest_folder)
    
    for root, _, files in os.walk(dest_folder):
        for file in files:
            if file.endswith(".json"):
                file_path = os.path.join(root, file)
                with open(file_path, "r", encoding="utf-8") as f:
                    try:
                        data = json.load(f)
                        cleared_data = clear_values(data)
                        with open(file_path, "w", encoding="utf-8") as f_out:
                            json.dump(cleared_data, f_out, ensure_ascii=False, indent=4)
                    except json.JSONDecodeError:
                        print(f"Warning: Could not parse {file_path}")
def main():
    if len(sys.argv) != 2:
        print("Usage: python script.py <new_language_code>")
        sys.exit(1)
    
    dest_language = sys.argv[1]
    clear_translation_keys(dest_language)
    print(f"Successfully created '{dest_language}' with empty translations.")

if __name__ == "__main__":
    main()